@@ -64,17 +64,18 @@ typedef struct
   DBusHashTable *entries;
 } BusServiceDirectory;
 
-typedef struct
+struct BusActivationEntry
 {
   int refcount;
   char *name;
   char *exec;
   char *user;
   char *systemd_service;
+  char *assumed_apparmor_label;
   unsigned long mtime;
   BusServiceDirectory *s_dir;
   char *filename;
-} BusActivationEntry;
+};
 
 typedef struct BusPendingActivationEntry BusPendingActivationEntry;
 
@@ -244,6 +245,7 @@ bus_activation_entry_unref (BusActivationEntry *entry)
   dbus_free (entry->user);
   dbus_free (entry->filename);
   dbus_free (entry->systemd_service);
+  dbus_free (entry->assumed_apparmor_label);
 
   dbus_free (entry);
 }
@@ -256,11 +258,13 @@ update_desktop_file_entry (BusActivation       *activation,
                            DBusError           *error)
 {
   char *name, *exec, *user, *exec_tmp, *systemd_service;
+  char *assumed_apparmor_label;
   BusActivationEntry *entry;
   DBusStat stat_buf;
   DBusString file_path;
   DBusError tmp_error;
   dbus_bool_t retval;
+  DBusString str;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -271,6 +275,7 @@ update_desktop_file_entry (BusActivation       *activation,
   exec_tmp = NULL;
   entry = NULL;
   systemd_service = NULL;
+  assumed_apparmor_label = NULL;
 
   dbus_error_init (&tmp_error);
 
@@ -308,9 +313,18 @@ update_desktop_file_entry (BusActivation       *activation,
                                     error))
     goto out;
 
-  exec = _dbus_strdup (_dbus_replace_install_prefix (exec_tmp));
-  dbus_free (exec_tmp);
-  exec_tmp = NULL;
+  if (!_dbus_string_init (&str))
+    goto out;
+
+  if (!_dbus_string_append (&str, exec_tmp) ||
+      !_dbus_replace_install_prefix (&str) ||
+      !_dbus_string_steal_data (&str, &exec))
+    {
+      _dbus_string_free (&str);
+      goto out;
+    }
+
+  _dbus_string_free (&str);
 
   /* user is not _required_ unless we are using system activation */
   if (!bus_desktop_file_get_string (desktop_file,
@@ -357,6 +371,28 @@ update_desktop_file_entry (BusActivation       *activation,
         }
     }
 
+  /* assumed AppArmor label is never required */
+  if (!bus_desktop_file_get_string (desktop_file,
+                                    DBUS_SERVICE_SECTION,
+                                    DBUS_SERVICE_ASSUMED_APPARMOR_LABEL,
+                                    &assumed_apparmor_label, &tmp_error))
+    {
+      _DBUS_ASSERT_ERROR_IS_SET (&tmp_error);
+      /* if we got OOM, then exit */
+      if (dbus_error_has_name (&tmp_error, DBUS_ERROR_NO_MEMORY))
+        {
+          dbus_move_error (&tmp_error, error);
+          goto out;
+        }
+      else
+        {
+          /* if we have error because we didn't find anything then continue */
+          dbus_error_free (&tmp_error);
+          dbus_free (assumed_apparmor_label);
+          assumed_apparmor_label = NULL;
+        }
+    }
+
   _DBUS_ASSERT_ERROR_IS_CLEAR (&tmp_error);
 
   entry = _dbus_hash_table_lookup_string (s_dir->entries,
@@ -385,6 +421,7 @@ update_desktop_file_entry (BusActivation       *activation,
       entry->exec = exec;
       entry->user = user;
       entry->systemd_service = systemd_service;
+      entry->assumed_apparmor_label = assumed_apparmor_label;
       entry->refcount = 1;
 
       /* ownership has been transferred to entry, do not free separately */
@@ -392,6 +429,7 @@ update_desktop_file_entry (BusActivation       *activation,
       exec = NULL;
       user = NULL;
       systemd_service = NULL;
+      assumed_apparmor_label = NULL;
 
       entry->s_dir = s_dir;
       entry->filename = _dbus_strdup (_dbus_string_get_const_data (filename));
@@ -449,6 +487,10 @@ update_desktop_file_entry (BusActivation       *activation,
       entry->systemd_service = systemd_service;
       systemd_service = NULL;
 
+      dbus_free (entry->assumed_apparmor_label);
+      entry->assumed_apparmor_label = assumed_apparmor_label;
+      assumed_apparmor_label = NULL;
+
       if (!_dbus_hash_table_insert_string (activation->entries,
                                            entry->name, bus_activation_entry_ref(entry)))
         {
@@ -466,10 +508,12 @@ update_desktop_file_entry (BusActivation       *activation,
 
 out:
   /* if these have been transferred into entry, the variables will be NULL */
+  dbus_free (exec_tmp);
   dbus_free (name);
   dbus_free (exec);
   dbus_free (user);
   dbus_free (systemd_service);
+  dbus_free (assumed_apparmor_label);
   _dbus_string_free (&file_path);
 
   if (entry)
@@ -729,9 +773,6 @@ update_directory (BusActivation       *activation,
 static dbus_bool_t
 populate_environment (BusActivation *activation)
 {
-  DBusString   key;
-  DBusString   value;
-  int          i;
   char       **environment;
   dbus_bool_t  retval = FALSE;
 
@@ -740,50 +781,7 @@ populate_environment (BusActivation *activation)
   if (environment == NULL)
     return FALSE;
 
-  if (!_dbus_string_init (&key))
-    {
-        dbus_free_string_array (environment);
-        return FALSE;
-    }
-
-  if (!_dbus_string_init (&value))
-    {
-      _dbus_string_free (&key);
-      dbus_free_string_array (environment);
-      return FALSE;
-    }
-
-  for (i = 0; environment[i] != NULL; i++)
-    {
-      if (!_dbus_string_append (&key, environment[i]))
-        break;
-
-      if (_dbus_string_split_on_byte (&key, '=', &value))
-        {
-          char *hash_key, *hash_value;
-
-          if (!_dbus_string_steal_data (&key, &hash_key))
-            break;
-
-          if (!_dbus_string_steal_data (&value, &hash_value))
-            break;
-
-          if (!_dbus_hash_table_insert_string (activation->environment,
-                                               hash_key, hash_value))
-            break;
-        }
-      _dbus_string_set_length (&key, 0);
-      _dbus_string_set_length (&value, 0);
-    }
-
-  if (environment[i] != NULL)
-    goto out;
-
-  retval = TRUE;
-out:
-
-  _dbus_string_free (&key);
-  _dbus_string_free (&value);
+  retval = _dbus_hash_table_from_array (activation->environment, environment, '=');
   dbus_free_string_array (environment);
 
   return retval;
@@ -1206,7 +1204,6 @@ bus_activation_send_pending_auto_activation_messages (BusActivation  *activation
                                                  entry->activation_message);
                 }
 
-              dbus_error_free (&error);
               link = next;
               continue;
             }
@@ -1469,7 +1466,12 @@ static dbus_bool_t
 pending_activation_timed_out (void *data)
 {
   BusPendingActivation *pending_activation = data;
+  BusContext *context;
   DBusError error;
+  int timeout;
+
+  context = pending_activation->activation->context;
+  timeout = bus_context_get_activation_timeout (context);
 
   /* Kill the spawned process, since it sucks
    * (not sure this is what we want to do, but
@@ -1480,10 +1482,11 @@ pending_activation_timed_out (void *data)
 
   dbus_error_init (&error);
 
-  bus_context_log_and_set_error (pending_activation->activation->context,
-                   DBUS_SYSTEM_LOG_INFO, &error, DBUS_ERROR_TIMED_OUT,
-                   "Failed to activate service '%s': timed out",
-                   pending_activation->service_name);
+  bus_context_log_and_set_error (context, DBUS_SYSTEM_LOG_WARNING, &error,
+                   DBUS_ERROR_TIMED_OUT,
+                   "Failed to activate service '%s': timed out "
+                   "(service_start_timeout=%dms)",
+                   pending_activation->service_name, timeout);
 
   pending_activation_failed (pending_activation, &error);
 
@@ -1601,51 +1604,7 @@ activation_find_entry (BusActivation *activation,
 static char **
 bus_activation_get_environment (BusActivation *activation)
 {
-  char **environment;
-  int i, length;
-  DBusString entry;
-  DBusHashIter iter;
-
-  length = _dbus_hash_table_get_n_entries (activation->environment);
-
-  environment = dbus_new0 (char *, length + 1);
-
-  if (environment == NULL)
-    return NULL;
-
-  i = 0;
-  _dbus_hash_iter_init (activation->environment, &iter);
-
-  if (!_dbus_string_init (&entry))
-    {
-      dbus_free_string_array (environment);
-      return NULL;
-    }
-
-  while (_dbus_hash_iter_next (&iter))
-    {
-      const char *key, *value;
-
-      key = (const char *) _dbus_hash_iter_get_string_key (&iter);
-      value = (const char *) _dbus_hash_iter_get_value (&iter);
-
-      if (!_dbus_string_append_printf (&entry, "%s=%s", key, value))
-        break;
-
-      if (!_dbus_string_steal_data (&entry, environment + i))
-        break;
-      i++;
-    }
-
-  _dbus_string_free (&entry);
-
-  if (i != length)
-    {
-      dbus_free_string_array (environment);
-      environment = NULL;
-    }
-
-  return environment;
+  return _dbus_hash_table_to_array (activation->environment, '=');
 }
 
 dbus_bool_t
@@ -1734,15 +1693,20 @@ bus_activation_activate_service (BusActivation  *activation,
   dbus_bool_t retval;
   dbus_bool_t was_pending_activation;
   DBusString command;
+  int limit;
+  DBusSpawnFlags flags = DBUS_SPAWN_NONE;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
-  if (activation->n_pending_activations >=
-      bus_context_get_max_pending_activations (activation->context))
+  limit = bus_context_get_max_pending_activations (activation->context);
+
+  if (activation->n_pending_activations >= limit)
     {
       dbus_set_error (error, DBUS_ERROR_LIMITS_EXCEEDED,
-                      "The maximum number of pending activations has been reached, activation of %s failed",
-                      service_name);
+                      "The maximum number of pending activations has been "
+                      "reached, activation of %s failed "
+                      "(max_pending_service_starts=%d)",
+                      service_name, limit);
       return FALSE;
     }
 
@@ -1760,6 +1724,24 @@ bus_activation_activate_service (BusActivation  *activation,
         return FALSE;
     }
 
+  if (auto_activation &&
+      entry != NULL &&
+      !bus_context_check_security_policy (activation->context,
+        transaction,
+        connection, /* sender */
+        NULL, /* addressed recipient */
+        NULL, /* proposed recipient */
+        activation_message,
+        entry,
+        error))
+    {
+      _DBUS_ASSERT_ERROR_IS_SET (error);
+      _dbus_verbose ("activation not authorized: %s: %s\n",
+          error != NULL ? error->name : "(error ignored)",
+          error != NULL ? error->message : "(error ignored)");
+      return FALSE;
+    }
+
   /* Bypass the registry lookup if we're auto-activating, bus_dispatch would not
    * call us if the service is already active.
    */
@@ -2034,9 +2016,11 @@ bus_activation_activate_service (BusActivation  *activation,
           if (service != NULL)
             {
               bus_context_log (activation->context,
-                               DBUS_SYSTEM_LOG_INFO, "Activating via systemd: service name='%s' unit='%s'",
+                               DBUS_SYSTEM_LOG_INFO, "Activating via systemd: service name='%s' unit='%s' requested by '%s' (%s)",
                                service_name,
-                               entry->systemd_service);
+                               entry->systemd_service,
+                               bus_connection_get_name (connection),
+                               bus_connection_get_loginfo (connection));
               /* Wonderful, systemd is connected, let's just send the msg */
               retval = bus_dispatch_matches (activation_transaction, NULL,
                                              systemd, message, error);
@@ -2044,9 +2028,11 @@ bus_activation_activate_service (BusActivation  *activation,
           else
             {
               bus_context_log (activation->context,
-                               DBUS_SYSTEM_LOG_INFO, "Activating systemd to hand-off: service name='%s' unit='%s'",
+                               DBUS_SYSTEM_LOG_INFO, "Activating systemd to hand-off: service name='%s' unit='%s' requested by '%s' (%s)",
                                service_name,
-                               entry->systemd_service);
+                               entry->systemd_service,
+                               bus_connection_get_name (connection),
+                               bus_connection_get_loginfo (connection));
               /* systemd is not around, let's "activate" it. */
               retval = bus_activation_activate_service (activation, NULL, activation_transaction, TRUE,
                                                         message, "org.freedesktop.systemd1", error);
@@ -2158,19 +2144,27 @@ bus_activation_activate_service (BusActivation  *activation,
   _dbus_verbose ("Spawning %s ...\n", argv[0]);
   if (servicehelper != NULL)
     bus_context_log (activation->context,
-                     DBUS_SYSTEM_LOG_INFO, "Activating service name='%s' (using servicehelper)",
-                     service_name);
+                     DBUS_SYSTEM_LOG_INFO, "Activating service name='%s' requested by '%s' (%s) (using servicehelper)",
+                     service_name,
+                     bus_connection_get_name (connection),
+                     bus_connection_get_loginfo (connection));
   else
     bus_context_log (activation->context,
-                     DBUS_SYSTEM_LOG_INFO, "Activating service name='%s'",
-                     service_name);
+                     DBUS_SYSTEM_LOG_INFO, "Activating service name='%s' requested by '%s' (%s)",
+                     service_name,
+                     bus_connection_get_name (connection),
+                     bus_connection_get_loginfo (connection));
 
   dbus_error_init (&tmp_error);
 
+  if (bus_context_get_using_syslog (activation->context))
+    flags |= DBUS_SPAWN_REDIRECT_OUTPUT;
+
   if (!_dbus_spawn_async_with_babysitter (&pending_activation->babysitter,
                                           service_name,
                                           argv,
                                           envp,
+                                          flags,
                                           child_setup,
                                           activation,
                                           &tmp_error))
@@ -2308,6 +2302,12 @@ dbus_activation_systemd_failure (BusActivation *activation,
   return TRUE;
 }
 
+const char *
+bus_activation_entry_get_assumed_apparmor_label (BusActivationEntry *entry)
+{
+  return entry->assumed_apparmor_label;
+}
+
 #ifdef DBUS_ENABLE_EMBEDDED_TESTS
 
 #include <stdio.h>
@@ -2441,8 +2441,21 @@ out:
 static dbus_bool_t
 init_service_reload_test (DBusString *dir)
 {
-  if (!_dbus_create_directory (dir, NULL))
-    return FALSE;
+  DBusStat stat_buf;
+
+  if (!_dbus_stat (dir, &stat_buf, NULL))
+    {
+      if (!_dbus_create_directory (dir, NULL))
+        return FALSE;
+    }
+  else
+    {
+      if (!test_remove_directory (dir))
+        return FALSE;
+
+      if (!_dbus_create_directory (dir, NULL))
+        return FALSE;
+    }
 
   /* Create one initial file */
   if (!test_create_service_file (dir, SERVICE_FILE_1, SERVICE_NAME_1, "exec-1"))
@@ -2630,9 +2643,6 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
       /* Do nothing? */
     }
 
-  if (!cleanup_service_reload_test (&directory))
-    goto out;
-
   /* Do OOM tests */
   if (!init_service_reload_test (&directory))
     _dbus_assert_not_reached ("could not initiate service reload test");
