@@ -370,14 +370,14 @@ check_return_values (const DBusString *full_path)
   user = bus_config_parser_get_user (parser);
   if (user == NULL)
     {
-      _dbus_warn ("User was NULL!\n");
+      _dbus_warn ("User was NULL!");
       goto finish;
     }
 #if 0
   /* the username can be configured in configure.in so this test doesn't work */
   if (strcmp (user, "dbus") != 0)
     {
-      _dbus_warn ("User was invalid; '%s'!\n", user);
+      _dbus_warn ("User was invalid; '%s'!", user);
       goto finish;
     }
   printf ("    <user>dbus</user> OKAY!\n");  
@@ -387,12 +387,12 @@ check_return_values (const DBusString *full_path)
   type = bus_config_parser_get_type (parser);
   if (type == NULL)
     {
-      _dbus_warn ("Type was NULL!\n");
+      _dbus_warn ("Type was NULL!");
       goto finish;
     }
   if (strcmp (type, "system") != 0)
     {
-      _dbus_warn ("Type was invalid; '%s'!\n", user);
+      _dbus_warn ("Type was invalid; '%s'!", user);
       goto finish;
     }
   printf ("    <type>system</type> OKAY!\n");
@@ -401,7 +401,7 @@ check_return_values (const DBusString *full_path)
   dirs = bus_config_parser_get_service_dirs (parser);
   if (dirs == NULL)
     {
-      _dbus_warn ("Service dirs are NULL!\n");
+      _dbus_warn ("Service dirs are NULL!");
       goto finish;
     }
   printf ("    <standard_system_service_dirs/> OKAY!\n");
@@ -440,7 +440,7 @@ do_load (const DBusString *full_path,
         }
       else if (validity == VALID)
         {
-          _dbus_warn ("Failed to load valid file but still had memory: %s\n",
+          _dbus_warn ("Failed to load valid file but still had memory: %s",
                       error.message);
 
           dbus_error_free (&error);
@@ -460,7 +460,7 @@ do_load (const DBusString *full_path,
 
       if (validity == INVALID)
         {
-          _dbus_warn ("Accepted invalid file\n");
+          _dbus_warn ("Accepted invalid file");
           return FALSE;
         }
 
@@ -497,7 +497,7 @@ process_test_valid_subdir (const DBusString *test_base_dir,
   dir = NULL;
 
   if (!_dbus_string_init (&test_directory))
-    _dbus_assert_not_reached ("didn't allocate test_directory\n");
+    _dbus_assert_not_reached ("didn't allocate test_directory");
 
   _dbus_string_init_const (&filename, subdir);
 
@@ -510,13 +510,13 @@ process_test_valid_subdir (const DBusString *test_base_dir,
 
   _dbus_string_free (&filename);
   if (!_dbus_string_init (&filename))
-    _dbus_assert_not_reached ("didn't allocate filename string\n");
+    _dbus_assert_not_reached ("didn't allocate filename string");
 
   dbus_error_init (&error);
   dir = _dbus_directory_open (&test_directory, &error);
   if (dir == NULL)
     {
-      _dbus_warn ("Could not open %s: %s\n",
+      _dbus_warn ("Could not open %s: %s",
                   _dbus_string_get_const_data (&test_directory),
                   error.message);
       dbus_error_free (&error);
@@ -576,7 +576,7 @@ process_test_valid_subdir (const DBusString *test_base_dir,
 
   if (dbus_error_is_set (&error))
     {
-      _dbus_warn ("Could not get next file in %s: %s\n",
+      _dbus_warn ("Could not get next file in %s: %s",
                   _dbus_string_get_const_data (&test_directory),
                   error.message);
       dbus_error_free (&error);
@@ -684,6 +684,13 @@ bus_config_parser_trivial_test (const DBusString *test_data_dir)
   if (!process_test_valid_subdir (test_data_dir, "valid-config-files", VALID))
     goto finish;
 
+#ifndef DBUS_WIN
+  /* We already test default_session_servicedirs and default_system_servicedirs
+   * in bus_config_parser_test() */
+  if (!process_test_valid_subdir (test_data_dir, "valid-config-files-system", VALID))
+    goto finish;
+#endif
+
   /* we don't process all the invalid files, as the trivial parser can't hope
    * to validate them all for all different syntaxes. We just check one broken
    * file to see if junk is received */
@@ -693,14 +700,16 @@ bus_config_parser_trivial_test (const DBusString *test_data_dir)
     goto finish;
   _dbus_string_free (&full_path);
 
+#ifndef DBUS_WIN
   /* just test if the check_file_valid works okay and we got sane values */
-  if (!make_full_path (test_data_dir, "valid-config-files", "system.conf", &full_path))
+  if (!make_full_path (test_data_dir, "valid-config-files-system", "system.conf", &full_path))
     goto finish;
   if (!check_file_valid (&full_path, VALID))
     goto finish;
   /* check to see if we got the correct values from the parser */
   if (!check_return_values (&full_path))
     goto finish;
+#endif
 
   /* woot! */
   retval = TRUE;
