@@ -428,6 +428,9 @@ bus_driver_handle_hello (DBusConnection *connection,
   dbus_bool_t retval;
   BusRegistry *registry;
   BusConnections *connections;
+  DBusError tmp_error;
+  int limit;
+  const char *limit_name;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -445,11 +448,19 @@ bus_driver_handle_hello (DBusConnection *connection,
    * incomplete connections. It's even OK if the connection wants to
    * retry the hello message, we support that.
    */
+  dbus_error_init (&tmp_error);
   connections = bus_connection_get_connections (connection);
   if (!bus_connections_check_limits (connections, connection,
-                                     error))
+                                     &limit_name, &limit,
+                                     &tmp_error))
     {
-      _DBUS_ASSERT_ERROR_IS_SET (error);
+      BusContext *context;
+
+      _DBUS_ASSERT_ERROR_IS_SET (&tmp_error);
+      context = bus_connection_get_context (connection);
+      bus_context_log (context, DBUS_SYSTEM_LOG_WARNING, "%s (%s=%d)",
+          tmp_error.message, limit_name, limit);
+      dbus_move_error (&tmp_error, error);
       return FALSE;
     }
 
@@ -1301,6 +1312,7 @@ bus_driver_handle_add_match (DBusConnection *connection,
   const char *text, *bustype;
   DBusString str;
   BusMatchmaker *matchmaker;
+  int limit;
   BusContext *context;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
@@ -1308,15 +1320,25 @@ bus_driver_handle_add_match (DBusConnection *connection,
   text = NULL;
   rule = NULL;
 
-  if (bus_connection_get_n_match_rules (connection) >=
-      bus_context_get_max_match_rules_per_connection (bus_transaction_get_context (transaction)))
+  context = bus_transaction_get_context (transaction);
+  limit = bus_context_get_max_match_rules_per_connection (context);
+
+  if (bus_connection_get_n_match_rules (connection) >= limit)
     {
-      dbus_set_error (error, DBUS_ERROR_LIMITS_EXCEEDED,
+      DBusError tmp_error;
+
+      dbus_error_init (&tmp_error);
+      dbus_set_error (&tmp_error, DBUS_ERROR_LIMITS_EXCEEDED,
                       "Connection \"%s\" is not allowed to add more match rules "
-                      "(increase limits in configuration file if required)",
+                      "(increase limits in configuration file if required; "
+                      "max_match_rules_per_connection=%d)",
                       bus_connection_is_active (connection) ?
                       bus_connection_get_name (connection) :
-                      "(inactive)");
+                      "(inactive)",
+                      limit);
+      bus_context_log (context, DBUS_SYSTEM_LOG_WARNING, "%s",
+                       tmp_error.message);
+      dbus_move_error (&tmp_error, error);
       goto failed;
     }
 
@@ -1334,7 +1356,6 @@ bus_driver_handle_add_match (DBusConnection *connection,
   if (rule == NULL)
     goto failed;
 
-  context = bus_transaction_get_context (transaction);
   bustype = context ? bus_context_get_type (context) : NULL;
   if (bus_match_rule_get_client_is_eavesdropping (rule) &&
       !bus_apparmor_allows_eavesdropping (connection, bustype, error))
@@ -1507,6 +1528,8 @@ bus_driver_handle_list_queued_owners (DBusConnection *connection,
 				      DBusMessage    *message,
 				      DBusError      *error)
 {
+  static const char dbus_service_name[] = DBUS_SERVICE_DBUS;
+
   const char *text;
   DBusList *base_names;
   DBusList *link;
@@ -1515,7 +1538,6 @@ bus_driver_handle_list_queued_owners (DBusConnection *connection,
   BusService *service;
   DBusMessage *reply;
   DBusMessageIter iter, array_iter;
-  char *dbus_service_name = DBUS_SERVICE_DBUS;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
@@ -1536,7 +1558,7 @@ bus_driver_handle_list_queued_owners (DBusConnection *connection,
       _dbus_string_equal_c_str (&str, DBUS_SERVICE_DBUS))
     {
       /* ORG_FREEDESKTOP_DBUS owns itself */
-      if (! _dbus_list_append (&base_names, dbus_service_name))
+      if (! _dbus_list_append (&base_names, (char *) dbus_service_name))
         goto oom;
     }
   else if (service == NULL)
@@ -1751,7 +1773,7 @@ bus_driver_handle_get_adt_audit_session_data (DBusConnection *connection,
   DBusConnection *conn;
   DBusMessage *reply;
   void *data = NULL;
-  dbus_uint32_t data_size;
+  dbus_int32_t data_size;
   const char *service;
   BusDriverFound found;
 
