@@ -39,12 +39,20 @@
 #include <unistd.h>     /* for write() and STDERR_FILENO */
 #endif
 #include "selinux.h"
+#include "apparmor.h"
+#include "audit.h"
+
+#ifdef DBUS_UNIX
+#include <dbus/dbus-sysdeps-unix.h>
+#endif
 
 static BusContext *context;
 
 #ifdef DBUS_UNIX
 
-static int reload_pipe[2];
+/* Despite its name and its unidirectional nature, this is actually
+ * a socket pair. */
+static DBusSocket reload_pipe[2];
 #define RELOAD_READ_END 0
 #define RELOAD_WRITE_END 1
 
@@ -67,7 +75,7 @@ signal_handler (int sig)
         char action[2] = { ACTION_RELOAD, '\0' };
 
         _dbus_string_init_const (&str, action);
-        if ((reload_pipe[RELOAD_WRITE_END] > 0) &&
+        if ((reload_pipe[RELOAD_WRITE_END].fd > 0) &&
             !_dbus_write_socket (reload_pipe[RELOAD_WRITE_END], &str, 0, 1))
           {
             /* If we receive SIGHUP often enough to fill the pipe buffer (4096
@@ -86,7 +94,8 @@ signal_handler (int sig)
             static const char message[] =
               "Unable to write to reload pipe - buffer full?\n";
 
-            if (write (STDERR_FILENO, message, strlen (message)) != strlen (message))
+            if (write (STDERR_FILENO, message, strlen (message)) !=
+                (ssize_t) strlen (message))
               {
                 /* ignore failure to write out a warning */
               }
@@ -99,7 +108,7 @@ signal_handler (int sig)
         DBusString str;
         char action[2] = { ACTION_QUIT, '\0' };
         _dbus_string_init_const (&str, action);
-        if ((reload_pipe[RELOAD_WRITE_END] < 0) ||
+        if ((reload_pipe[RELOAD_WRITE_END].fd < 0) ||
             !_dbus_write_socket (reload_pipe[RELOAD_WRITE_END], &str, 0, 1))
           {
             /* If we can't write to the socket, dying seems a more
@@ -110,7 +119,8 @@ signal_handler (int sig)
               "Unable to write termination signal to pipe - buffer full?\n"
               "Will exit instead.\n";
 
-            if (write (STDERR_FILENO, message, strlen (message)) != strlen (message))
+            if (write (STDERR_FILENO, message, strlen (message)) !=
+                (ssize_t) strlen (message))
               {
                 /* ignore failure to write out a warning */
               }
@@ -122,6 +132,8 @@ signal_handler (int sig)
 }
 #endif /* DBUS_UNIX */
 
+static void usage (void) _DBUS_GNUC_NORETURN;
+
 static void
 usage (void)
 {
@@ -136,6 +148,9 @@ usage (void)
       " [--introspect]"
       " [--address=ADDRESS]"
       " [--nopidfile]"
+      " [--nosyslog]"
+      " [--syslog]"
+      " [--syslog-only]"
       " [--nofork]"
 #ifdef DBUS_UNIX
       " [--fork]"
@@ -145,6 +160,8 @@ usage (void)
   exit (1);
 }
 
+static void version (void) _DBUS_GNUC_NORETURN;
+
 static void
 version (void)
 {
@@ -156,6 +173,8 @@ version (void)
   exit (0);
 }
 
+static void introspect (void) _DBUS_GNUC_NORETURN;
+
 static void
 introspect (void)
 {
@@ -177,7 +196,7 @@ introspect (void)
   exit (0);
 
  oom:
-  _dbus_warn ("Can not introspect - Out of memory\n");
+  _dbus_warn ("Can not introspect - Out of memory");
   exit (1);
 }
 
@@ -243,10 +262,10 @@ handle_reload_watch (DBusWatch    *watch,
   while (!_dbus_string_init (&str))
     _dbus_wait_for_memory ();
 
-  if ((reload_pipe[RELOAD_READ_END] > 0) &&
+  if ((reload_pipe[RELOAD_READ_END].fd > 0) &&
       _dbus_read_socket (reload_pipe[RELOAD_READ_END], &str, 1) != 1)
     {
-      _dbus_warn ("Couldn't read from reload pipe.\n");
+      _dbus_warn ("Couldn't read from reload pipe.");
       close_reload_pipe (&watch);
       return TRUE;
     }
@@ -272,7 +291,7 @@ handle_reload_watch (DBusWatch    *watch,
           _DBUS_ASSERT_ERROR_IS_SET (&error);
           _dbus_assert (dbus_error_has_name (&error, DBUS_ERROR_FAILED) ||
                         dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY));
-          _dbus_warn ("Unable to reload configuration: %s\n",
+          _dbus_warn ("Unable to reload configuration: %s",
                       error.message);
           dbus_error_free (&error);
         }
@@ -310,22 +329,22 @@ setup_reload_pipe (DBusLoop *loop)
 
   dbus_error_init (&error);
 
-  if (!_dbus_full_duplex_pipe (&reload_pipe[0], &reload_pipe[1],
-			       TRUE, &error))
+  if (!_dbus_socketpair (&reload_pipe[0], &reload_pipe[1],
+                         TRUE, &error))
     {
-      _dbus_warn ("Unable to create reload pipe: %s\n",
+      _dbus_warn ("Unable to create reload pipe: %s",
 		  error.message);
       dbus_error_free (&error);
       exit (1);
     }
 
-  watch = _dbus_watch_new (reload_pipe[RELOAD_READ_END],
-			   DBUS_WATCH_READABLE, TRUE,
-			   handle_reload_watch, NULL, NULL);
+  watch = _dbus_watch_new (_dbus_socket_get_pollable (reload_pipe[RELOAD_READ_END]),
+                           DBUS_WATCH_READABLE, TRUE,
+                           handle_reload_watch, NULL, NULL);
 
   if (watch == NULL)
     {
-      _dbus_warn ("Unable to create reload watch: %s\n",
+      _dbus_warn ("Unable to create reload watch: %s",
 		  error.message);
       dbus_error_free (&error);
       exit (1);
@@ -333,7 +352,7 @@ setup_reload_pipe (DBusLoop *loop)
 
   if (!_dbus_loop_add_watch (loop, watch))
     {
-      _dbus_warn ("Unable to add reload watch to main loop: %s\n",
+      _dbus_warn ("Unable to add reload watch to main loop: %s",
 		  error.message);
       dbus_error_free (&error);
       exit (1);
@@ -350,10 +369,10 @@ close_reload_pipe (DBusWatch **watch)
     *watch = NULL;
 
     _dbus_close_socket (reload_pipe[RELOAD_READ_END], NULL);
-    reload_pipe[RELOAD_READ_END] = -1;
+    _dbus_socket_invalidate (&reload_pipe[RELOAD_READ_END]);
 
     _dbus_close_socket (reload_pipe[RELOAD_WRITE_END], NULL);
-    reload_pipe[RELOAD_WRITE_END] = -1;
+    _dbus_socket_invalidate (&reload_pipe[RELOAD_WRITE_END]);
 }
 #endif /* DBUS_UNIX */
 
@@ -372,6 +391,27 @@ main (int argc, char **argv)
   dbus_bool_t print_address;
   dbus_bool_t print_pid;
   BusContextFlags flags;
+#ifdef DBUS_UNIX
+  const char *error_str;
+
+  /* Redirect stdin from /dev/null since we will never need it, and
+   * redirect stdout and stderr to /dev/null if not already open.
+   *
+   * We should do this as the very first thing, to ensure that when we
+   * create other file descriptors (for example for epoll, inotify or
+   * a socket), they never get assigned as fd 0, 1 or 2. If they were,
+   * which could happen if our caller had (incorrectly) closed those
+   * standard fds, they'd get closed when we daemonize - for example,
+   * closing our listening socket would stop us listening, and closing
+   * a Linux epoll socket would cause the main loop to fail. */
+  if (!_dbus_ensure_standard_fds (DBUS_FORCE_STDIN_NULL, &error_str))
+    {
+      fprintf (stderr,
+               "dbus-daemon: fatal error setting up standard fds: %s: %s\n",
+               error_str, _dbus_strerror (errno));
+      return 1;
+    }
+#endif
 
   if (!_dbus_string_init (&config_file))
     return 1;
@@ -410,6 +450,21 @@ main (int argc, char **argv)
         {
           introspect ();
         }
+      else if (strcmp (arg, "--nosyslog") == 0)
+        {
+          flags &= ~BUS_CONTEXT_FLAG_SYSLOG_ALWAYS;
+          flags |= BUS_CONTEXT_FLAG_SYSLOG_NEVER;
+        }
+      else if (strcmp (arg, "--syslog") == 0)
+        {
+          flags &= ~BUS_CONTEXT_FLAG_SYSLOG_NEVER;
+          flags |= BUS_CONTEXT_FLAG_SYSLOG_ALWAYS;
+        }
+      else if (strcmp (arg, "--syslog-only") == 0)
+        {
+          flags &= ~BUS_CONTEXT_FLAG_SYSLOG_NEVER;
+          flags |= (BUS_CONTEXT_FLAG_SYSLOG_ALWAYS|BUS_CONTEXT_FLAG_SYSLOG_ONLY);
+        }
       else if (strcmp (arg, "--nofork") == 0)
         {
           flags &= ~BUS_CONTEXT_FLAG_FORK_ALWAYS;
@@ -434,14 +489,14 @@ main (int argc, char **argv)
         {
           check_two_config_files (&config_file, "system");
 
-          if (!_dbus_append_system_config_file (&config_file))
+          if (!_dbus_get_system_config_file (&config_file))
             exit (1);
         }
       else if (strcmp (arg, "--session") == 0)
         {
           check_two_config_files (&config_file, "session");
 
-          if (!_dbus_append_session_config_file (&config_file))
+          if (!_dbus_get_session_config_file (&config_file))
             exit (1);
         }
       else if (strstr (arg, "--config-file=") == arg)
@@ -610,7 +665,13 @@ main (int argc, char **argv)
 
   if (!bus_selinux_pre_init ())
     {
-      _dbus_warn ("SELinux pre-initialization failed\n");
+      _dbus_warn ("SELinux pre-initialization failed");
+      exit (1);
+    }
+
+  if (!bus_apparmor_pre_init ())
+    {
+      _dbus_warn ("AppArmor pre-initialization failed: out of memory");
       exit (1);
     }
 
@@ -622,7 +683,7 @@ main (int argc, char **argv)
   _dbus_string_free (&config_file);
   if (context == NULL)
     {
-      _dbus_warn ("Failed to start message bus: %s\n",
+      _dbus_warn ("Failed to start message bus: %s",
                   error.message);
       dbus_error_free (&error);
       exit (1);
@@ -649,6 +710,8 @@ main (int argc, char **argv)
   bus_context_shutdown (context);
   bus_context_unref (context);
   bus_selinux_shutdown ();
+  bus_apparmor_shutdown ();
+  bus_audit_shutdown ();
 
   return 0;
 }
