@@ -16,7 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -61,12 +61,10 @@ dbus_bool_t bus_selinux_allows_send            (DBusConnection *sender,
 						const char     *member,
 						const char     *error_name,
 						const char     *destination,
+						BusActivationEntry *activation_entry,
 						DBusError      *error);
 
 BusSELinuxID* bus_selinux_init_connection_id (DBusConnection *connection,
                                               DBusError      *error);
 
-
-void bus_selinux_audit_init(void);
-
 #endif /* BUS_SELINUX_H */
