@@ -38,7 +38,8 @@ typedef enum
   DBUS_AUTH_STATE_WAITING_FOR_MEMORY,
   DBUS_AUTH_STATE_HAVE_BYTES_TO_SEND,
   DBUS_AUTH_STATE_NEED_DISCONNECT,
-  DBUS_AUTH_STATE_AUTHENTICATED
+  DBUS_AUTH_STATE_AUTHENTICATED,
+  DBUS_AUTH_STATE_INVALID = -1
 } DBusAuthState;
 
 DBUS_PRIVATE_EXPORT
