@@ -83,9 +83,9 @@ _dbus_validate_signature_with_reason (const DBusString *type_str,
       goto out;
     }
 
-  p = _dbus_string_get_const_data_len (type_str, type_pos, 0);
+  p = _dbus_string_get_const_udata_len (type_str, type_pos, 0);
 
-  end = _dbus_string_get_const_data_len (type_str, type_pos + len, 0);
+  end = _dbus_string_get_const_udata_len (type_str, type_pos + len, 0);
   struct_depth = 0;
   array_depth = 0;
   dict_entry_depth = 0;
@@ -233,7 +233,7 @@ _dbus_validate_signature_with_reason (const DBusString *type_str,
         {
           if (*p == DBUS_TYPE_ARRAY && p != end)
             {
-	       const char *p1;
+               const unsigned char *p1;
 	       p1 = p + 1;
                if (*p1 == DBUS_STRUCT_END_CHAR ||
                    *p1 == DBUS_DICT_ENTRY_END_CHAR)
@@ -358,13 +358,8 @@ validate_body_helper (DBusTypeReader       *reader,
           
           if (current_type == DBUS_TYPE_BOOLEAN)
             {
-              dbus_uint32_t v;
-
-              if (p + 4 > end)
-                return DBUS_INVALID_NOT_ENOUGH_DATA;
-
-              v = _dbus_unpack_uint32 (byte_order, p);
-
+              dbus_uint32_t v = _dbus_unpack_uint32 (byte_order,
+                                                     p);
               if (!(v == 0 || v == 1))
                 return DBUS_INVALID_BOOLEAN_NOT_ZERO_OR_ONE;
             }
@@ -425,7 +420,7 @@ validate_body_helper (DBusTypeReader       *reader,
             if (current_type == DBUS_TYPE_OBJECT_PATH)
               {
                 DBusString str;
-                _dbus_string_init_const_len (&str, p, claimed_len);
+                _dbus_string_init_const_len (&str, (const char *) p, claimed_len);
                 if (!_dbus_validate_path (&str, 0,
                                           _dbus_string_get_length (&str)))
                   return DBUS_INVALID_BAD_PATH;
@@ -435,7 +430,7 @@ validate_body_helper (DBusTypeReader       *reader,
             else if (current_type == DBUS_TYPE_STRING)
               {
                 DBusString str;
-                _dbus_string_init_const_len (&str, p, claimed_len);
+                _dbus_string_init_const_len (&str, (const char *) p, claimed_len);
                 if (!_dbus_string_validate_utf8 (&str, 0,
                                                  _dbus_string_get_length (&str)))
                   return DBUS_INVALID_BAD_UTF8_IN_STRING;
@@ -534,7 +529,7 @@ validate_body_helper (DBusTypeReader       *reader,
             if (claimed_len + 1 > (unsigned long) (end - p))
               return DBUS_INVALID_SIGNATURE_LENGTH_OUT_OF_BOUNDS;
 
-            _dbus_string_init_const_len (&str, p, claimed_len);
+            _dbus_string_init_const_len (&str, (const char *) p, claimed_len);
             validity =
               _dbus_validate_signature_with_reason (&str, 0,
                                                     _dbus_string_get_length (&str));
@@ -578,7 +573,7 @@ validate_body_helper (DBusTypeReader       *reader,
             if (claimed_len + 1 > (unsigned long) (end - p))
               return DBUS_INVALID_VARIANT_SIGNATURE_LENGTH_OUT_OF_BOUNDS;
 
-            _dbus_string_init_const_len (&sig, p, claimed_len);
+            _dbus_string_init_const_len (&sig, (const char *) p, claimed_len);
             reason = _dbus_validate_signature_with_reason (&sig, 0,
                                            _dbus_string_get_length (&sig));
             if (!(reason == DBUS_VALID))
@@ -730,7 +725,7 @@ _dbus_validate_body_with_reason (const DBusString *expected_signature,
   _dbus_type_reader_init_types_only (&reader,
                                      expected_signature, expected_signature_start);
 
-  p = _dbus_string_get_const_data_len (value_str, value_pos, len);
+  p = _dbus_string_get_const_udata_len (value_str, value_pos, len);
   end = p + len;
 
   validity = validate_body_helper (&reader, byte_order, TRUE, 0, p, end, &p);
@@ -805,7 +800,7 @@ _dbus_validate_path (const DBusString  *str,
   if (len == 0)
     return FALSE;
 
-  s = _dbus_string_get_const_data (str) + start;
+  s = _dbus_string_get_const_udata (str) + start;
   end = s + len;
 
   if (*s != '/')
@@ -946,7 +941,7 @@ _dbus_validate_interface (const DBusString  *str,
     return FALSE;
 
   last_dot = NULL;
-  iface = _dbus_string_get_const_data (str) + start;
+  iface = _dbus_string_get_const_udata (str) + start;
   end = iface + len;
   s = iface;
 
@@ -1020,7 +1015,7 @@ _dbus_validate_member (const DBusString  *str,
   if (len == 0)
     return FALSE;
 
-  member = _dbus_string_get_const_data (str) + start;
+  member = _dbus_string_get_const_udata (str) + start;
   end = member + len;
   s = member;
 
@@ -1112,7 +1107,7 @@ _dbus_validate_bus_name_full (const DBusString  *str,
     return FALSE;
 
   last_dot = NULL;
-  iface = _dbus_string_get_const_data (str) + start;
+  iface = _dbus_string_get_const_udata (str) + start;
   end = iface + len;
   s = iface;
 
