@@ -97,7 +97,7 @@
  * @{
  */
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 static dbus_bool_t debug_initialized = FALSE;
 static int fail_nth = -1;
 static size_t fail_size = 0;
@@ -240,7 +240,7 @@ _dbus_get_fail_alloc_failures (void)
   return n_failures_per_failure;
 }
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 /**
  * Called when about to alloc some memory; if
  * it returns #TRUE, then the allocation should
@@ -294,7 +294,7 @@ _dbus_decrement_fail_alloc_counter (void)
       return FALSE;
     }
 }
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
 
 /**
  * Get the number of outstanding malloc()'d blocks.
@@ -364,7 +364,7 @@ check_guards (void       *free_block,
           dbus_uint32_t value = *(dbus_uint32_t*) &block[i];
           if (value != GUARD_VALUE)
             {
-              _dbus_warn ("Block of %lu bytes from %s had start guard value 0x%ux at %d expected 0x%x\n",
+              _dbus_warn ("Block of %lu bytes from %s had start guard value 0x%ux at %d expected 0x%x",
                           (long) requested_bytes, source_string (source),
                           value, i, GUARD_VALUE);
               failed = TRUE;
@@ -379,7 +379,7 @@ check_guards (void       *free_block,
           dbus_uint32_t value = *(dbus_uint32_t*) &block[i];
           if (value != GUARD_VALUE)
             {
-              _dbus_warn ("Block of %lu bytes from %s had end guard value 0x%ux at %d expected 0x%x\n",
+              _dbus_warn ("Block of %lu bytes from %s had end guard value 0x%ux at %d expected 0x%x",
                           (long) requested_bytes, source_string (source),
                           value, i, GUARD_VALUE);
               failed = TRUE;
@@ -460,7 +460,7 @@ set_guards (void       *real_block,
 void*
 dbus_malloc (size_t bytes)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   _dbus_initialize_malloc_debug ();
   
   if (_dbus_decrement_fail_alloc_counter ())
@@ -472,7 +472,7 @@ dbus_malloc (size_t bytes)
 
   if (bytes == 0) /* some system mallocs handle this, some don't */
     return NULL;
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   else if (fail_size != 0 && bytes > fail_size)
     return NULL;
   else if (guards)
@@ -486,7 +486,7 @@ dbus_malloc (size_t bytes)
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: malloc (%ld + %ld)\n",
+          _dbus_warn ("out of memory: malloc (%ld + %ld)",
               (long) bytes, (long) GUARD_EXTRA_SIZE);
           _dbus_abort ();
         }
@@ -499,14 +499,14 @@ dbus_malloc (size_t bytes)
       void *mem;
       mem = malloc (bytes);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (mem)
         {
           _dbus_atomic_inc (&n_blocks_outstanding);
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: malloc (%ld)\n", (long) bytes);
+          _dbus_warn ("out of memory: malloc (%ld)", (long) bytes);
           _dbus_abort ();
         }
 #endif
@@ -530,7 +530,7 @@ dbus_malloc (size_t bytes)
 void*
 dbus_malloc0 (size_t bytes)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   _dbus_initialize_malloc_debug ();
   
   if (_dbus_decrement_fail_alloc_counter ())
@@ -543,7 +543,7 @@ dbus_malloc0 (size_t bytes)
   
   if (bytes == 0)
     return NULL;
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   else if (fail_size != 0 && bytes > fail_size)
     return NULL;
   else if (guards)
@@ -558,7 +558,7 @@ dbus_malloc0 (size_t bytes)
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: calloc (%ld + %ld, 1)\n",
+          _dbus_warn ("out of memory: calloc (%ld + %ld, 1)",
               (long) bytes, (long) GUARD_EXTRA_SIZE);
           _dbus_abort ();
         }
@@ -571,14 +571,14 @@ dbus_malloc0 (size_t bytes)
       void *mem;
       mem = calloc (bytes, 1);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (mem)
         {
           _dbus_atomic_inc (&n_blocks_outstanding);
         }
       else if (malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: calloc (%ld)\n", (long) bytes);
+          _dbus_warn ("out of memory: calloc (%ld)", (long) bytes);
           _dbus_abort ();
         }
 #endif
@@ -601,7 +601,7 @@ void*
 dbus_realloc (void  *memory,
               size_t bytes)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   _dbus_initialize_malloc_debug ();
   
   if (_dbus_decrement_fail_alloc_counter ())
@@ -617,7 +617,7 @@ dbus_realloc (void  *memory,
       dbus_free (memory);
       return NULL;
     }
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   else if (fail_size != 0 && bytes > fail_size)
     return NULL;
   else if (guards)
@@ -636,7 +636,7 @@ dbus_realloc (void  *memory,
             {
               if (malloc_cannot_fail)
                 {
-                  _dbus_warn ("out of memory: realloc (%p, %ld + %ld)\n",
+                  _dbus_warn ("out of memory: realloc (%p, %ld + %ld)",
                       memory, (long) bytes, (long) GUARD_EXTRA_SIZE);
                   _dbus_abort ();
                 }
@@ -663,7 +663,7 @@ dbus_realloc (void  *memory,
             }
           else if (malloc_cannot_fail)
             {
-              _dbus_warn ("out of memory: malloc (%ld + %ld)\n",
+              _dbus_warn ("out of memory: malloc (%ld + %ld)",
                   (long) bytes, (long) GUARD_EXTRA_SIZE);
               _dbus_abort ();
             }
@@ -677,10 +677,10 @@ dbus_realloc (void  *memory,
       void *mem;
       mem = realloc (memory, bytes);
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
       if (mem == NULL && malloc_cannot_fail)
         {
-          _dbus_warn ("out of memory: malloc (%ld)\n", (long) bytes);
+          _dbus_warn ("out of memory: malloc (%ld)", (long) bytes);
           _dbus_abort ();
         }
 
@@ -700,7 +700,7 @@ dbus_realloc (void  *memory,
 void
 dbus_free (void  *memory)
 {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
   if (guards)
     {
       check_guards (memory, TRUE);
@@ -724,7 +724,7 @@ dbus_free (void  *memory)
     
   if (memory) /* we guarantee it's safe to free (NULL) */
     {
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #ifdef DBUS_DISABLE_ASSERT
       _dbus_atomic_dec (&n_blocks_outstanding);
 #else
@@ -795,7 +795,7 @@ struct ShutdownClosure
   void *data;                /**< Data for function */
 };
 
-_DBUS_DEFINE_GLOBAL_LOCK (shutdown_funcs);
+/* Protected by _DBUS_LOCK (shutdown_funcs) */
 static ShutdownClosure *registered_globals = NULL;
 
 /**
@@ -809,6 +809,20 @@ static ShutdownClosure *registered_globals = NULL;
 dbus_bool_t
 _dbus_register_shutdown_func (DBusShutdownFunction  func,
                               void                 *data)
+{
+  dbus_bool_t ok;
+
+  if (!_DBUS_LOCK (shutdown_funcs))
+    return FALSE;
+
+  ok = _dbus_register_shutdown_func_unlocked (func, data);
+  _DBUS_UNLOCK (shutdown_funcs);
+  return ok;
+}
+
+dbus_bool_t
+_dbus_register_shutdown_func_unlocked (DBusShutdownFunction  func,
+                                       void                 *data)
 {
   ShutdownClosure *c;
 
@@ -820,13 +834,9 @@ _dbus_register_shutdown_func (DBusShutdownFunction  func,
   c->func = func;
   c->data = data;
 
-  _DBUS_LOCK (shutdown_funcs);
-  
   c->next = registered_globals;
   registered_globals = c;
 
-  _DBUS_UNLOCK (shutdown_funcs);
-  
   return TRUE;
 }
 
@@ -846,12 +856,19 @@ _dbus_register_shutdown_func (DBusShutdownFunction  func,
  * can be useful to free these internal data structures.
  *
  * dbus_shutdown() does NOT free memory that was returned
- * to the application. It only returns libdbus-internal
+ * to the application. It only frees libdbus-internal
  * data structures.
  *
  * You MUST free all memory and release all reference counts
  * returned to you by libdbus prior to calling dbus_shutdown().
  *
+ * If a shared connection is open, calling dbus_shutdown() will
+ * drain its queue of messages and disconnect it. In particular,
+ * this will result in processing of the special Disconnected
+ * signal, which may result in a call to _exit(), unless you
+ * have used dbus_connection_set_exit_on_disconnect() to disable
+ * that behaviour.
+ *
  * You can't continue to use any D-Bus objects, such as connections,
  * that were allocated prior to dbus_shutdown(). You can, however,
  * start over; call dbus_threads_init() again, create new connections,
@@ -902,7 +919,7 @@ dbus_shutdown (void)
 
 /** @} */ /** End of public API docs block */
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include "dbus-test.h"
 
 /**
