@@ -45,7 +45,12 @@
 #include <systemd/sd-journal.h>
 #endif
 
+#if defined(__APPLE__)
+# include <crt_externs.h>
+# define environ (*_NSGetEnviron ())
+#elif !HAVE_DECL_ENVIRON
 extern char **environ;
+#endif
 
 /**
  * @addtogroup DBusInternalsUtils
@@ -243,7 +248,7 @@ struct DBusBabysitter
   char *log_name; /**< the name under which to log messages about this
 		    process being spawned */
   
-  int socket_to_babysitter; /**< Connection to the babysitter process */
+  DBusSocket socket_to_babysitter; /**< Connection to the babysitter process */
   int error_pipe_from_child; /**< Connection to the process that does the exec() */
   
   pid_t sitter_pid;  /**< PID Of the babysitter */
@@ -275,7 +280,7 @@ _dbus_babysitter_new (void)
 
   sitter->refcount = 1;
 
-  sitter->socket_to_babysitter = -1;
+  sitter->socket_to_babysitter.fd = -1;
   sitter->error_pipe_from_child = -1;
   
   sitter->sitter_pid = -1;
@@ -369,9 +374,9 @@ _dbus_babysitter_unref (DBusBabysitter *sitter)
           if (ret < 0)
             {
               if (errno == ECHILD)
-                _dbus_warn ("Babysitter process not available to be reaped; should not happen\n");
+                _dbus_warn ("Babysitter process not available to be reaped; should not happen");
               else
-                _dbus_warn ("Unexpected error %d in waitpid() for babysitter: %s\n",
+                _dbus_warn ("Unexpected error %d in waitpid() for babysitter: %s",
                             errno, _dbus_strerror (errno));
             }
           else
@@ -415,7 +420,7 @@ read_data (DBusBabysitter *sitter,
   switch (r)
     {
     case READ_STATUS_ERROR:
-      _dbus_warn ("Failed to read data from fd %d: %s\n", fd, error.message);
+      _dbus_warn ("Failed to read data from fd %d: %s", fd, error.message);
       dbus_error_free (&error);
       return r;
 
@@ -441,7 +446,7 @@ read_data (DBusBabysitter *sitter,
             switch (r)
               {
               case READ_STATUS_ERROR:
-                _dbus_warn ("Failed to read arg from fd %d: %s\n", fd, error.message);
+                _dbus_warn ("Failed to read arg from fd %d: %s", fd, error.message);
                 dbus_error_free (&error);
                 return r;
               case READ_STATUS_EOF:
@@ -454,9 +459,25 @@ read_data (DBusBabysitter *sitter,
               {
                 if (what == CHILD_EXITED)
                   {
+                    /* Do not reset sitter->errnum to 0 here. We get here if
+                     * the babysitter reports that the grandchild process has
+                     * exited, and there are two ways that can happen:
+                     *
+                     * 1. grandchild successfully exec()s the desired process,
+                     * but then the desired process exits or is terminated
+                     * by a signal. The babysitter observes this and reports
+                     * CHILD_EXITED.
+                     *
+                     * 2. grandchild fails to exec() the desired process,
+                     * attempts to report the exec() failure (which
+                     * we will receive as CHILD_EXEC_FAILED), and then
+                     * exits itself (which will prompt the babysitter to
+                     * send CHILD_EXITED). We want the CHILD_EXEC_FAILED
+                     * to take precedence (and have its errno logged),
+                     * which _dbus_babysitter_set_child_exit_error() does.
+                     */
                     sitter->have_child_status = TRUE;
                     sitter->status = arg;
-                    sitter->errnum = 0;
                     _dbus_verbose ("recorded child status exited = %d signaled = %d exitstatus = %d termsig = %d\n",
                                    WIFEXITED (sitter->status), WIFSIGNALED (sitter->status),
                                    WEXITSTATUS (sitter->status), WTERMSIG (sitter->status));
@@ -485,7 +506,7 @@ read_data (DBusBabysitter *sitter,
             switch (r)
               {
               case READ_STATUS_ERROR:
-                _dbus_warn ("Failed to read PID from fd %d: %s\n", fd, error.message);
+                _dbus_warn ("Failed to read PID from fd %d: %s", fd, error.message);
                 dbus_error_free (&error);
                 return r;
               case READ_STATUS_EOF:
@@ -500,7 +521,7 @@ read_data (DBusBabysitter *sitter,
           }
           break;
         default:
-          _dbus_warn ("Unknown message received from babysitter process\n");
+          _dbus_warn ("Unknown message received from babysitter process");
           break;
         }
     }
@@ -522,10 +543,10 @@ close_socket_to_babysitter (DBusBabysitter *sitter)
       sitter->sitter_watch = NULL;
     }
 
-  if (sitter->socket_to_babysitter >= 0)
+  if (sitter->socket_to_babysitter.fd >= 0)
     {
       _dbus_close_socket (sitter->socket_to_babysitter, NULL);
-      sitter->socket_to_babysitter = -1;
+      sitter->socket_to_babysitter.fd = -1;
     }
 }
 
@@ -545,7 +566,7 @@ close_error_pipe_from_child (DBusBabysitter *sitter)
 
   if (sitter->error_pipe_from_child >= 0)
     {
-      _dbus_close_socket (sitter->error_pipe_from_child, NULL);
+      _dbus_close (sitter->error_pipe_from_child, NULL);
       sitter->error_pipe_from_child = -1;
     }
 }
@@ -561,7 +582,7 @@ handle_babysitter_socket (DBusBabysitter *sitter,
   if (revents & _DBUS_POLLIN)
     {
       _dbus_verbose ("Reading data from babysitter\n");
-      if (read_data (sitter, sitter->socket_to_babysitter) != READ_STATUS_OK)
+      if (read_data (sitter, sitter->socket_to_babysitter.fd) != READ_STATUS_OK)
         close_socket_to_babysitter (sitter);
     }
   else if (revents & (_DBUS_POLLERR | _DBUS_POLLHUP))
@@ -607,9 +628,9 @@ babysitter_iteration (DBusBabysitter *sitter,
       ++i;
     }
   
-  if (sitter->socket_to_babysitter >= 0)
+  if (sitter->socket_to_babysitter.fd >= 0)
     {
-      fds[i].fd = sitter->socket_to_babysitter;
+      fds[i].fd = sitter->socket_to_babysitter.fd;
       fds[i].events = _DBUS_POLLIN;
       fds[i].revents = 0;
       ++i;
@@ -643,7 +664,7 @@ babysitter_iteration (DBusBabysitter *sitter,
               --i;
               if (fds[i].fd == sitter->error_pipe_from_child)
                 handle_error_pipe (sitter, fds[i].revents);
-              else if (fds[i].fd == sitter->socket_to_babysitter)
+              else if (fds[i].fd == sitter->socket_to_babysitter.fd)
                 handle_babysitter_socket (sitter, fds[i].revents);
             }
         }
@@ -656,7 +677,7 @@ babysitter_iteration (DBusBabysitter *sitter,
  * Macro returns #TRUE if the babysitter still has live sockets open to the
  * babysitter child or the grandchild.
  */
-#define LIVE_CHILDREN(sitter) ((sitter)->socket_to_babysitter >= 0 || (sitter)->error_pipe_from_child >= 0)
+#define LIVE_CHILDREN(sitter) ((sitter)->socket_to_babysitter.fd >= 0 || (sitter)->error_pipe_from_child >= 0)
 
 /**
  * Blocks until the babysitter process gives us the PID of the spawned grandchild,
@@ -696,7 +717,7 @@ _dbus_babysitter_get_child_exited (DBusBabysitter *sitter)
     ;
 
   /* We will have exited the babysitter when the child has exited */
-  return sitter->socket_to_babysitter < 0;
+  return sitter->socket_to_babysitter.fd < 0;
 }
 
 /**
@@ -830,7 +851,7 @@ handle_watch (DBusWatch       *watch,
 
   if (fd == sitter->error_pipe_from_child)
     handle_error_pipe (sitter, revents);
-  else if (fd == sitter->socket_to_babysitter)
+  else if (fd == sitter->socket_to_babysitter.fd)
     handle_babysitter_socket (sitter, revents);
 
   while (LIVE_CHILDREN (sitter) &&
@@ -839,7 +860,7 @@ handle_watch (DBusWatch       *watch,
 
   /* fd.o #32992: if the handle_* methods closed their sockets, they previously
    * didn't always remove the watches. Check that we don't regress. */
-  _dbus_assert (sitter->socket_to_babysitter != -1 || sitter->sitter_watch == NULL);
+  _dbus_assert (sitter->socket_to_babysitter.fd != -1 || sitter->sitter_watch == NULL);
   _dbus_assert (sitter->error_pipe_from_child != -1 || sitter->error_watch == NULL);
 
   if (_dbus_babysitter_get_child_exited (sitter) &&
@@ -877,7 +898,7 @@ close_and_invalidate (int *fd)
     return -1;
   else
     {
-      ret = _dbus_close_socket (*fd, NULL);
+      ret = _dbus_close (*fd, NULL);
       *fd = -1;
     }
 
@@ -944,7 +965,7 @@ do_write (int fd, const void *buf, size_t count)
         goto again;
       else
         {
-          _dbus_warn ("Failed to write data to pipe!\n");
+          _dbus_warn ("Failed to write data to pipe!");
           exit (1); /* give up, we suck */
         }
     }
@@ -955,6 +976,8 @@ do_write (int fd, const void *buf, size_t count)
     goto again;
 }
 
+static void write_err_and_exit (int fd, int msg) _DBUS_GNUC_NORETURN;
+
 static void
 write_err_and_exit (int fd, int msg)
 {
@@ -975,6 +998,8 @@ write_pid (int fd, pid_t pid)
   do_write (fd, &pid, sizeof (pid));
 }
 
+static void write_status_and_exit (int fd, int status) _DBUS_GNUC_NORETURN;
+
 static void
 write_status_and_exit (int fd, int status)
 {
@@ -986,10 +1011,16 @@ write_status_and_exit (int fd, int status)
   exit (0);
 }
 
+static void do_exec (int                       child_err_report_fd,
+                     char             * const *argv,
+                     char             * const *envp,
+                     DBusSpawnChildSetupFunc   child_setup,
+                     void                     *user_data) _DBUS_GNUC_NORETURN;
+
 static void
 do_exec (int                       child_err_report_fd,
-	 char                    **argv,
-	 char                    **envp,
+	 char             * const *argv,
+	 char             * const *envp,
 	 DBusSpawnChildSetupFunc   child_setup,
 	 void                     *user_data)
 {
@@ -1017,7 +1048,7 @@ do_exec (int                       child_err_report_fd,
       retval = fcntl (i, F_GETFD);
 
       if (retval != -1 && !(retval & FD_CLOEXEC))
-	_dbus_warn ("Fd %d did not have the close-on-exec flag set!\n", i);
+        _dbus_warn ("Fd %d did not have the close-on-exec flag set!", i);
     }
 #endif
 
@@ -1061,7 +1092,7 @@ check_babysit_events (pid_t grandchild_pid,
   else if (ret < 0)
     {
       /* This isn't supposed to happen. */
-      _dbus_warn ("unexpected waitpid() failure in check_babysit_events(): %s\n",
+      _dbus_warn ("unexpected waitpid() failure in check_babysit_events(): %s",
                   _dbus_strerror (errno));
       exit (1);
     }
@@ -1074,7 +1105,7 @@ check_babysit_events (pid_t grandchild_pid,
     }
   else
     {
-      _dbus_warn ("waitpid() reaped pid %d that we've never heard of\n",
+      _dbus_warn ("waitpid() reaped pid %d that we've never heard of",
                   (int) ret);
       exit (1);
     }
@@ -1104,6 +1135,9 @@ babysit_signal_handler (int signo)
       goto again;
 }
 
+static void babysit (pid_t grandchild_pid,
+                     int   parent_pipe) _DBUS_GNUC_NORETURN;
+
 static void
 babysit (pid_t grandchild_pid,
          int   parent_pipe)
@@ -1122,7 +1156,7 @@ babysit (pid_t grandchild_pid,
    */
   if (pipe (sigchld_pipe) < 0)
     {
-      _dbus_warn ("Not enough file descriptors to create pipe in babysitter process\n");
+      _dbus_warn ("Not enough file descriptors to create pipe in babysitter process");
       exit (1);
     }
 
@@ -1148,7 +1182,7 @@ babysit (pid_t grandchild_pid,
       
       if (_dbus_poll (pfds, _DBUS_N_ELEMENTS (pfds), -1) < 0 && errno != EINTR)
         {
-          _dbus_warn ("_dbus_poll() error: %s\n", strerror (errno));
+          _dbus_warn ("_dbus_poll() error: %s", strerror (errno));
           exit (1);
         }
 
@@ -1172,9 +1206,14 @@ babysit (pid_t grandchild_pid,
 }
 
 /**
- * Spawns a new process. The child_setup
- * function is passed the given user_data and is run in the child
- * just before calling exec().
+ * Spawns a new process.
+ *
+ * On Unix platforms, the child_setup function is passed the given
+ * user_data and is run in the child after fork() but before calling exec().
+ * This can be used to change uid, resource limits and so on.
+ * On Windows, this functionality does not fit the multi-processing model
+ * (Windows does the equivalent of fork() and exec() in a single API call),
+ * and the child_setup function and its user_data are ignored.
  *
  * Also creates a "babysitter" which tracks the status of the
  * child process, advising the parent if the child exits.
@@ -1193,15 +1232,16 @@ babysit (pid_t grandchild_pid,
 dbus_bool_t
 _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
                                    const char               *log_name,
-                                   char                    **argv,
+                                   char             * const *argv,
                                    char                    **env,
+                                   DBusSpawnFlags            flags,
                                    DBusSpawnChildSetupFunc   child_setup,
                                    void                     *user_data,
                                    DBusError                *error)
 {
   DBusBabysitter *sitter;
   int child_err_report_pipe[2] = { -1, -1 };
-  int babysitter_pipe[2] = { -1, -1 };
+  DBusSocket babysitter_pipe[2] = { DBUS_SOCKET_INIT, DBUS_SOCKET_INIT };
   pid_t pid;
 #ifdef HAVE_SYSTEMD
   int fd_out = -1;
@@ -1242,7 +1282,7 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
   if (!make_pipe (child_err_report_pipe, error))
     goto cleanup_and_fail;
 
-  if (!_dbus_full_duplex_pipe (&babysitter_pipe[0], &babysitter_pipe[1], TRUE, error))
+  if (!_dbus_socketpair (&babysitter_pipe[0], &babysitter_pipe[1], TRUE, error))
     goto cleanup_and_fail;
 
   /* Setting up the babysitter is only useful in the parent,
@@ -1271,7 +1311,7 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
       goto cleanup_and_fail;
     }
       
-  sitter->sitter_watch = _dbus_watch_new (babysitter_pipe[0],
+  sitter->sitter_watch = _dbus_watch_new (babysitter_pipe[0].fd,
                                           DBUS_WATCH_READABLE,
                                           TRUE, handle_watch, sitter, NULL);
   if (sitter->sitter_watch == NULL)
@@ -1295,12 +1335,15 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
 
 #ifdef HAVE_SYSTEMD
-  /* This may fail, but it's not critical.
-   * In particular, if we were compiled with journald support but are now
-   * running on a non-systemd system, this is going to fail, so we
-   * have to cope gracefully. */
-  fd_out = sd_journal_stream_fd (sitter->log_name, LOG_INFO, FALSE);
-  fd_err = sd_journal_stream_fd (sitter->log_name, LOG_WARNING, FALSE);
+  if (flags & DBUS_SPAWN_REDIRECT_OUTPUT)
+    {
+      /* This may fail, but it's not critical.
+       * In particular, if we were compiled with journald support but are now
+       * running on a non-systemd system, this is going to fail, so we
+       * have to cope gracefully. */
+      fd_out = sd_journal_stream_fd (sitter->log_name, LOG_INFO, FALSE);
+      fd_err = sd_journal_stream_fd (sitter->log_name, LOG_WARNING, FALSE);
+    }
 #endif
 
   pid = fork ();
@@ -1325,24 +1368,44 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
 
       /* Close the parent's end of the pipes. */
       close_and_invalidate (&child_err_report_pipe[READ_END]);
-      close_and_invalidate (&babysitter_pipe[0]);
+      close_and_invalidate (&babysitter_pipe[0].fd);
       
       /* Create the child that will exec () */
       grandchild_pid = fork ();
       
       if (grandchild_pid < 0)
 	{
-	  write_err_and_exit (babysitter_pipe[1],
+	  write_err_and_exit (babysitter_pipe[1].fd,
 			      CHILD_FORK_FAILED);
           _dbus_assert_not_reached ("Got to code after write_err_and_exit()");
 	}
       else if (grandchild_pid == 0)
       {
+#ifdef __linux__
+          int fd = -1;
+
+#ifdef O_CLOEXEC
+          fd = open ("/proc/self/oom_score_adj", O_WRONLY | O_CLOEXEC);
+#endif
+
+          if (fd < 0)
+            {
+              fd = open ("/proc/self/oom_score_adj", O_WRONLY);
+              _dbus_fd_set_close_on_exec (fd);
+            }
+
+          if (fd >= 0)
+            {
+              if (write (fd, "0", sizeof (char)) < 0)
+                _dbus_warn ("writing oom_score_adj error: %s", strerror (errno));
+              _dbus_close (fd, NULL);
+            }
+#endif
           /* Go back to ignoring SIGPIPE, since it's evil
            */
           signal (SIGPIPE, SIG_IGN);
 
-          close_and_invalidate (&babysitter_pipe[1]);
+          close_and_invalidate (&babysitter_pipe[1].fd);
 #ifdef HAVE_SYSTEMD
 	  /* log to systemd journal if possible */
 	  if (fd_out >= 0)
@@ -1365,7 +1428,7 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
           close_and_invalidate (&fd_out);
           close_and_invalidate (&fd_err);
 #endif
-          babysit (grandchild_pid, babysitter_pipe[1]);
+          babysit (grandchild_pid, babysitter_pipe[1].fd);
           _dbus_assert_not_reached ("Got to code after babysit()");
 	}
     }
@@ -1373,14 +1436,14 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
     {      
       /* Close the uncared-about ends of the pipes */
       close_and_invalidate (&child_err_report_pipe[WRITE_END]);
-      close_and_invalidate (&babysitter_pipe[1]);
+      close_and_invalidate (&babysitter_pipe[1].fd);
 #ifdef HAVE_SYSTEMD
       close_and_invalidate (&fd_out);
       close_and_invalidate (&fd_err);
 #endif
 
       sitter->socket_to_babysitter = babysitter_pipe[0];
-      babysitter_pipe[0] = -1;
+      babysitter_pipe[0].fd = -1;
       
       sitter->error_pipe_from_child = child_err_report_pipe[READ_END];
       child_err_report_pipe[READ_END] = -1;
@@ -1405,8 +1468,8 @@ _dbus_spawn_async_with_babysitter (DBusBabysitter          **sitter_p,
   
   close_and_invalidate (&child_err_report_pipe[READ_END]);
   close_and_invalidate (&child_err_report_pipe[WRITE_END]);
-  close_and_invalidate (&babysitter_pipe[0]);
-  close_and_invalidate (&babysitter_pipe[1]);
+  close_and_invalidate (&babysitter_pipe[0].fd);
+  close_and_invalidate (&babysitter_pipe[1].fd);
 #ifdef HAVE_SYSTEMD
   close_and_invalidate (&fd_out);
   close_and_invalidate (&fd_err);
@@ -1429,259 +1492,9 @@ _dbus_babysitter_set_result_function  (DBusBabysitter             *sitter,
 
 /** @} */
 
-#ifdef DBUS_ENABLE_EMBEDDED_TESTS
-
-static char *
-get_test_exec (const char *exe,
-               DBusString *scratch_space)
-{
-  const char *dbus_test_exec;
-
-  dbus_test_exec = _dbus_getenv ("DBUS_TEST_EXEC");
-
-  if (dbus_test_exec == NULL)
-    dbus_test_exec = DBUS_TEST_EXEC;
-
-  if (!_dbus_string_init (scratch_space))
-    return NULL;
-
-  if (!_dbus_string_append_printf (scratch_space, "%s/%s%s",
-                                   dbus_test_exec, exe, DBUS_EXEEXT))
-    {
-      _dbus_string_free (scratch_space);
-      return NULL;
-    }
-
-  return _dbus_string_get_data (scratch_space);
-}
-
-static void
+void
 _dbus_babysitter_block_for_child_exit (DBusBabysitter *sitter)
 {
   while (LIVE_CHILDREN (sitter))
     babysitter_iteration (sitter, TRUE);
 }
-
-static dbus_bool_t
-check_spawn_nonexistent (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter = NULL;
-  DBusError error = DBUS_ERROR_INIT;
-
-  /*** Test launching nonexistent binary */
-  
-  argv[0] = "/this/does/not/exist/32542sdgafgafdg";
-  if (_dbus_spawn_async_with_babysitter (&sitter, "spawn_nonexistent", argv,
-                                         NULL, NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_block_for_child_exit (sitter);
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error launching nonexistent executable\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_EXEC_FAILED)))
-    {
-      _dbus_warn ("Not expecting error when launching nonexistent executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-  
-  return TRUE;
-}
-
-static dbus_bool_t
-check_spawn_segfault (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter = NULL;
-  DBusError error = DBUS_ERROR_INIT;
-  DBusString argv0;
-
-  /*** Test launching segfault binary */
-
-  argv[0] = get_test_exec ("test-segfault", &argv0);
-
-  if (argv[0] == NULL)
-    {
-      /* OOM was simulated, never mind */
-      return TRUE;
-    }
-
-  if (_dbus_spawn_async_with_babysitter (&sitter, "spawn_segfault", argv,
-                                         NULL, NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_block_for_child_exit (sitter);
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  _dbus_string_free (&argv0);
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error launching segfaulting binary\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_CHILD_SIGNALED)))
-    {
-      _dbus_warn ("Not expecting error when launching segfaulting executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-  
-  return TRUE;
-}
-
-static dbus_bool_t
-check_spawn_exit (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter = NULL;
-  DBusError error = DBUS_ERROR_INIT;
-  DBusString argv0;
-
-  /*** Test launching exit failure binary */
-
-  argv[0] = get_test_exec ("test-exit", &argv0);
-
-  if (argv[0] == NULL)
-    {
-      /* OOM was simulated, never mind */
-      return TRUE;
-    }
-
-  if (_dbus_spawn_async_with_babysitter (&sitter, "spawn_exit", argv,
-                                         NULL, NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_block_for_child_exit (sitter);
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  _dbus_string_free (&argv0);
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error launching binary that exited with failure code\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_CHILD_EXITED)))
-    {
-      _dbus_warn ("Not expecting error when launching exiting executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-  
-  return TRUE;
-}
-
-static dbus_bool_t
-check_spawn_and_kill (void *data)
-{
-  char *argv[4] = { NULL, NULL, NULL, NULL };
-  DBusBabysitter *sitter = NULL;
-  DBusError error = DBUS_ERROR_INIT;
-  DBusString argv0;
-
-  /*** Test launching sleeping binary then killing it */
-
-  argv[0] = get_test_exec ("test-sleep-forever", &argv0);
-
-  if (argv[0] == NULL)
-    {
-      /* OOM was simulated, never mind */
-      return TRUE;
-    }
-
-  if (_dbus_spawn_async_with_babysitter (&sitter, "spawn_and_kill", argv,
-                                         NULL, NULL, NULL,
-                                         &error))
-    {
-      _dbus_babysitter_kill_child (sitter);
-      
-      _dbus_babysitter_block_for_child_exit (sitter);
-      
-      _dbus_babysitter_set_child_exit_error (sitter, &error);
-    }
-
-  _dbus_string_free (&argv0);
-
-  if (sitter)
-    _dbus_babysitter_unref (sitter);
-
-  if (!dbus_error_is_set (&error))
-    {
-      _dbus_warn ("Did not get an error after killing spawned binary\n");
-      return FALSE;
-    }
-
-  if (!(dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY) ||
-        dbus_error_has_name (&error, DBUS_ERROR_SPAWN_CHILD_SIGNALED)))
-    {
-      _dbus_warn ("Not expecting error when killing executable: %s: %s\n",
-                  error.name, error.message);
-      dbus_error_free (&error);
-      return FALSE;
-    }
-
-  dbus_error_free (&error);
-  
-  return TRUE;
-}
-
-dbus_bool_t
-_dbus_spawn_test (const char *test_data_dir)
-{
-  if (!_dbus_test_oom_handling ("spawn_nonexistent",
-                                check_spawn_nonexistent,
-                                NULL))
-    return FALSE;
-
-  if (!_dbus_test_oom_handling ("spawn_segfault",
-                                check_spawn_segfault,
-                                NULL))
-    return FALSE;
-
-  if (!_dbus_test_oom_handling ("spawn_exit",
-                                check_spawn_exit,
-                                NULL))
-    return FALSE;
-
-  if (!_dbus_test_oom_handling ("spawn_and_kill",
-                                check_spawn_and_kill,
-                                NULL))
-    return FALSE;
-  
-  return TRUE;
-}
-#endif
