@@ -92,6 +92,7 @@ typedef struct DBusPipe DBusPipe;
  * @{
  */
 
+DBUS_PRIVATE_EXPORT
 void _dbus_abort (void) _DBUS_GNUC_NORETURN;
 
 dbus_bool_t _dbus_check_setuid (void);
@@ -198,7 +199,7 @@ int _dbus_read_socket_with_unix_fds      (DBusSocket        fd,
                                           DBusString       *buffer,
                                           int               count,
                                           int              *fds,
-                                          int              *n_fds);
+                                          unsigned int     *n_fds);
 DBUS_PRIVATE_EXPORT
 int _dbus_write_socket_with_unix_fds     (DBusSocket        fd,
                                           const DBusString *buffer,
@@ -285,16 +286,6 @@ struct DBusAtomic
 #endif
 };
 
-/* The value we get from autofoo is in the form of a cpp expression;
- * convert that to a conventional defined/undef switch. (We can't get
- * the conventional defined/undef because of multiarch builds only running
- * ./configure once, on Darwin.) */
-#if DBUS_HAVE_ATOMIC_INT_COND
-#   define DBUS_HAVE_ATOMIC_INT 1
-#else
-#   undef DBUS_HAVE_ATOMIC_INT
-#endif
-
 DBUS_PRIVATE_EXPORT
 dbus_int32_t _dbus_atomic_inc (DBusAtomic *atomic);
 DBUS_PRIVATE_EXPORT
@@ -421,9 +412,6 @@ DBUS_PRIVATE_EXPORT
 dbus_bool_t    _dbus_create_directory        (const DBusString *filename,
                                               DBusError        *error);
 DBUS_PRIVATE_EXPORT
-dbus_bool_t    _dbus_ensure_directory        (const DBusString *filename,
-                                              DBusError        *error);
-DBUS_PRIVATE_EXPORT
 dbus_bool_t    _dbus_delete_directory        (const DBusString *filename,
 					      DBusError        *error);
 
@@ -438,8 +426,8 @@ dbus_bool_t _dbus_path_is_absolute    (const DBusString *filename);
 dbus_bool_t _dbus_get_standard_session_servicedirs (DBusList **dirs);
 dbus_bool_t _dbus_get_standard_system_servicedirs (DBusList **dirs);
 
-dbus_bool_t _dbus_append_system_config_file  (DBusString *str);
-dbus_bool_t _dbus_append_session_config_file (DBusString *str);
+dbus_bool_t _dbus_get_system_config_file  (DBusString *str);
+dbus_bool_t _dbus_get_session_config_file (DBusString *str);
 
 /** Opaque type for reading a directory listing */
 typedef struct DBusDirIter DBusDirIter;
@@ -495,8 +483,12 @@ void _dbus_exit (int code) _DBUS_GNUC_NORETURN;
 
 DBUS_PRIVATE_EXPORT
 int _dbus_printf_string_upper_bound (const char *format,
-                                     va_list args);
+                                     va_list args) _DBUS_GNUC_PRINTF (1, 0);
 
+#ifdef DBUS_ENABLE_VERBOSE_MODE
+DBUS_PRIVATE_EXPORT
+void _dbus_print_thread (void);
+#endif
 
 /**
  * Portable struct with stat() results
@@ -522,6 +514,7 @@ dbus_bool_t _dbus_socketpair (DBusSocket       *fd1,
                               dbus_bool_t       blocking,
                               DBusError        *error);
 
+DBUS_PRIVATE_EXPORT
 void        _dbus_print_backtrace  (void);
 
 dbus_bool_t _dbus_become_daemon   (const DBusString *pidfile,
@@ -552,7 +545,14 @@ void _dbus_set_signal_handler (int               sig,
 dbus_bool_t _dbus_user_at_console (const char *username,
                                    DBusError  *error);
 
-void _dbus_init_system_log (dbus_bool_t is_daemon);
+typedef enum {
+  DBUS_LOG_FLAGS_STDERR = (1 << 0),
+  DBUS_LOG_FLAGS_SYSTEM_LOG = (1 << 1)
+} DBusLogFlags;
+
+DBUS_PRIVATE_EXPORT
+void _dbus_init_system_log (const char   *tag,
+                            DBusLogFlags  flags);
 
 typedef enum {
   DBUS_SYSTEM_LOG_INFO,
@@ -561,8 +561,14 @@ typedef enum {
   DBUS_SYSTEM_LOG_FATAL
 } DBusSystemLogSeverity;
 
-void _dbus_system_log (DBusSystemLogSeverity severity, const char *msg, ...) _DBUS_GNUC_PRINTF (2, 3);
-void _dbus_system_logv (DBusSystemLogSeverity severity, const char *msg, va_list args);
+DBUS_PRIVATE_EXPORT
+void _dbus_log  (DBusSystemLogSeverity  severity,
+                 const char            *msg,
+                 ...) _DBUS_GNUC_PRINTF (2, 3);
+DBUS_PRIVATE_EXPORT
+void _dbus_logv (DBusSystemLogSeverity  severity,
+                 const char            *msg,
+                 va_list args) _DBUS_GNUC_PRINTF (2, 0);
 
 /* Define DBUS_VA_COPY() to do the right thing for copying va_list variables.
  * config.h may have already defined DBUS_VA_COPY as va_copy or __va_copy.
@@ -650,21 +656,10 @@ dbus_pid_t    _dbus_getpid (void);
 DBUS_PRIVATE_EXPORT
 dbus_uid_t    _dbus_getuid (void);
 
-dbus_bool_t _dbus_change_to_daemon_user (const char *user,
-                                         DBusError  *error);
-
 DBUS_PRIVATE_EXPORT
 void _dbus_flush_caches (void);
 
-/*
- * replaces the term DBUS_PREFIX in configure_time_path by the
- * current dbus installation directory. On unix this function is a noop
- *
- * @param configure_time_path
- * @return real path
- */
-const char *
-_dbus_replace_install_prefix (const char *configure_time_path);
+dbus_bool_t _dbus_replace_install_prefix (DBusString *path);
 
 /* Do not set this too high: it is a denial-of-service risk.
  * See <https://bugs.freedesktop.org/show_bug.cgi?id=82820>
@@ -683,16 +678,6 @@ dbus_bool_t     _dbus_rlimit_restore_fd_limit              (DBusRLimit   *saved,
                                                             DBusError    *error);
 void            _dbus_rlimit_free                          (DBusRLimit   *lim);
 
-#ifdef DBUS_ENABLE_EMBEDDED_TESTS
-_DBUS_GNUC_WARN_UNUSED_RESULT
-dbus_bool_t _dbus_test_append_different_uid (DBusString *uid);
-
-#ifdef DBUS_UNIX
-_DBUS_GNUC_WARN_UNUSED_RESULT
-dbus_bool_t _dbus_test_append_different_username (DBusString *username);
-#endif
-#endif  /* DBUS_ENABLE_EMBEDDED_TESTS */
-
 /** @} */
 
 DBUS_END_DECLS
