@@ -17,10 +17,11 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+#include <config.h>
 #include "dbus-internals.h"
 #include "dbus-timeout.h"
 #include "dbus-list.h"
@@ -261,12 +262,12 @@ _dbus_timeout_list_set_functions (DBusTimeoutList           *timeout_list,
               link2 = _dbus_list_get_first_link (&timeout_list->timeouts);
               while (link2 != link)
                 {
-                  DBusList *next = _dbus_list_get_next_link (&timeout_list->timeouts,
-                                                             link2);
+                  DBusList *next2 = _dbus_list_get_next_link (&timeout_list->timeouts,
+                                                              link2);
 
                   (* remove_function) (link2->data, data);
                   
-                  link2 = next;
+                  link2 = next2;
                 }
 
               return FALSE;
