@@ -63,6 +63,7 @@ live_messages_notify (DBusCounter *counter,
 {
   DBusTransport *transport = user_data;
 
+  _dbus_connection_lock (transport->connection);
   _dbus_transport_ref (transport);
 
 #if 0
@@ -77,12 +78,11 @@ live_messages_notify (DBusCounter *counter,
    */
   if (transport->vtable->live_messages_changed)
     {
-      _dbus_connection_lock (transport->connection);
       (* transport->vtable->live_messages_changed) (transport);
-      _dbus_connection_unlock (transport->connection);
     }
 
   _dbus_transport_unref (transport);
+  _dbus_connection_unlock (transport->connection);
 }
 
 /**
@@ -899,7 +899,7 @@ _dbus_transport_handle_watch (DBusTransport           *transport,
 
   if (dbus_watch_get_socket (watch) < 0)
     {
-      _dbus_warn_check_failed ("Tried to handle an invalidated watch; this watch should have been removed\n");
+      _dbus_warn_check_failed ("Tried to handle an invalidated watch; this watch should have been removed");
       return TRUE;
     }
   
@@ -949,7 +949,7 @@ _dbus_transport_set_connection (DBusTransport  *transport,
  */
 dbus_bool_t
 _dbus_transport_get_socket_fd (DBusTransport *transport,
-                               int           *fd_p)
+                               DBusSocket    *fd_p)
 {
   dbus_bool_t retval;
   
@@ -1055,14 +1055,18 @@ recover_unused_bytes (DBusTransport *transport)
     {
       const DBusString *bytes;
       DBusString *buffer;
+#ifdef DBUS_ENABLE_VERBOSE_MODE
       int orig_len;
+#endif
       dbus_bool_t succeeded;
 
       _dbus_message_loader_get_buffer (transport->loader,
                                        &buffer);
-                
+
+#ifdef DBUS_ENABLE_VERBOSE_MODE
       orig_len = _dbus_string_get_length (buffer);
-                
+#endif
+
       _dbus_auth_get_unused_bytes (transport->auth,
                                    &bytes);
 
@@ -1142,9 +1146,9 @@ _dbus_transport_queue_messages (DBusTransport *transport)
   DBusDispatchStatus status;
 
 #if 0
-  _dbus_verbose ("_dbus_transport_queue_messages()\n");
+  _dbus_verbose ("enter\n");
 #endif
-  
+
   /* Queue any messages */
   while ((status = _dbus_transport_get_dispatch_status (transport)) == DBUS_DISPATCH_DATA_REMAINS)
     {
