@@ -34,22 +34,19 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <gio/gio.h>
 
 #include "test-utils-glib.h"
 
 #include <string.h>
 
 #ifdef DBUS_UNIX
-# include <pwd.h>
 # include <unistd.h>
 # include <sys/types.h>
 
-# ifdef HAVE_SYS_RESOURCE_H
-#   include <sys/resource.h>
-# endif
-
-# ifdef HAVE_SYS_TIME_H
-#   include <sys/time.h>
+# ifdef HAVE_GIO_UNIX
+    /* The CMake build system doesn't know how to check for this yet */
+#   include <gio/gunixfdmessage.h>
 # endif
 #endif
 
@@ -87,11 +84,14 @@ typedef struct {
     GError *ge;
 
     GPid daemon_pid;
+    gchar *address;
 
     DBusConnection *left_conn;
 
     DBusConnection *right_conn;
+    GQueue held_messages;
     gboolean right_conn_echo;
+    gboolean right_conn_hold;
     gboolean wait_forever_called;
 
     gchar *tmp_runtime_dir;
@@ -129,11 +129,25 @@ echo_filter (DBusConnection *connection,
   return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+hold_filter (DBusConnection *connection,
+    DBusMessage *message,
+    void *user_data)
+{
+  Fixture *f = user_data;
+
+  if (dbus_message_get_type (message) != DBUS_MESSAGE_TYPE_METHOD_CALL)
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+  g_queue_push_tail (&f->held_messages, dbus_message_ref (message));
+
+  return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 typedef struct {
     const char *bug_ref;
     guint min_messages;
     const char *config_file;
-    TestUser user;
     enum { SPECIFY_ADDRESS = 0, RELY_ON_DEFAULT } connect_mode;
 } Config;
 
@@ -142,7 +156,6 @@ setup (Fixture *f,
     gconstpointer context)
 {
   const Config *config = context;
-  gchar *address;
 
   f->ctx = test_main_context_get ();
   f->ge = NULL;
@@ -157,20 +170,19 @@ setup (Fixture *f,
       /* we're relying on being single-threaded for this to be safe */
       f->saved_runtime_dir = g_strdup (g_getenv ("XDG_RUNTIME_DIR"));
       g_setenv ("XDG_RUNTIME_DIR", f->tmp_runtime_dir, TRUE);
-      g_unsetenv ("DBUS_SESSION_BUS_ADDRESS");
     }
 
-  address = test_get_dbus_daemon (config ? config->config_file : NULL,
-                                  config ? config->user : TEST_USER_ME,
-                                  &f->daemon_pid);
+  f->address = test_get_dbus_daemon (config ? config->config_file : NULL,
+                                     TEST_USER_ME,
+                                     &f->daemon_pid);
 
-  if (address == NULL)
+  if (f->address == NULL)
     {
       f->skip = TRUE;
       return;
     }
 
-  f->left_conn = test_connect_to_bus (f->ctx, address);
+  f->left_conn = test_connect_to_bus (f->ctx, f->address);
 
   if (config != NULL && config->connect_mode == RELY_ON_DEFAULT)
     {
@@ -184,10 +196,8 @@ setup (Fixture *f,
     }
   else
     {
-      f->right_conn = test_connect_to_bus (f->ctx, address);
+      f->right_conn = test_connect_to_bus (f->ctx, f->address);
     }
-
-  g_free (address);
 }
 
 static void
@@ -199,6 +209,15 @@ add_echo_filter (Fixture *f)
   f->right_conn_echo = TRUE;
 }
 
+static void
+add_hold_filter (Fixture *f)
+{
+  if (!dbus_connection_add_filter (f->right_conn, hold_filter, f, NULL))
+    g_error ("OOM");
+
+  f->right_conn_hold = TRUE;
+}
+
 static void
 pc_count (DBusPendingCall *pc,
     void *data)
@@ -208,6 +227,17 @@ pc_count (DBusPendingCall *pc,
   (*received_p)++;
 }
 
+static void
+pc_enqueue (DBusPendingCall *pc,
+            void *data)
+{
+  GQueue *q = data;
+  DBusMessage *m = dbus_pending_call_steal_reply (pc);
+
+  g_test_message ("message of type %d", dbus_message_get_type (m));
+  g_queue_push_tail (q, m);
+}
+
 static void
 test_echo (Fixture *f,
     gconstpointer context)
@@ -656,69 +686,354 @@ test_canonical_path_uae (Fixture *f,
   dbus_message_unref (m);
 }
 
-#define DESIRED_RLIMIT 65536
-
-#ifdef DBUS_UNIX
 static void
-test_fd_limit (Fixture *f,
-               gconstpointer context)
+test_max_connections (Fixture *f,
+    gconstpointer context)
 {
-#ifdef HAVE_PRLIMIT
-  struct rlimit lim;
-  const struct passwd *pwd = NULL;
-#endif
+  DBusError error = DBUS_ERROR_INIT;
+  DBusConnection *third_conn;
+  DBusConnection *failing_conn;
 
   if (f->skip)
     return;
 
-#ifdef HAVE_PRLIMIT
+  /* We have two connections already */
+  g_assert (f->left_conn != NULL);
+  g_assert (f->right_conn != NULL);
 
-  if (getuid () != 0)
+  /* Our configuration file sets the limit to 3 connections, either globally
+   * or per uid, so this one is the last that will work */
+  third_conn = test_connect_to_bus (f->ctx, f->address);
+
+  /* This one is going to fail. We don't guarantee whether it will fail
+   * now, or while registering (implementation detail: it's the latter). */
+  failing_conn = dbus_connection_open_private (f->address, &error);
+
+  if (failing_conn != NULL)
     {
-      g_test_skip ("Cannot test, only uid 0 is expected to raise fd limit");
-      return;
+      gboolean ok = dbus_bus_register (failing_conn, &error);
+
+      g_assert (!ok);
     }
 
-  pwd = getpwnam (DBUS_USER);
+  g_assert (dbus_error_is_set (&error));
+  g_assert_cmpstr (error.name, ==, DBUS_ERROR_LIMITS_EXCEEDED);
 
-  if (pwd == NULL)
+  if (failing_conn != NULL)
     {
-      gchar *message = g_strdup_printf ("user '%s' does not exist",
-          DBUS_USER);
-
-      g_test_skip (message);
-      g_free (message);
-      return;
+      dbus_connection_close (failing_conn);
+      dbus_connection_unref (failing_conn);
     }
 
-  if (prlimit (getpid (), RLIMIT_NOFILE, NULL, &lim) < 0)
-    g_error ("prlimit(): %s", g_strerror (errno));
-
-  g_test_message ("our RLIMIT_NOFILE: rlim_cur: %ld, rlim_max: %ld",
-                  (long) lim.rlim_cur, (long) lim.rlim_max);
-
-  if (lim.rlim_cur == RLIM_INFINITY || lim.rlim_cur >= DESIRED_RLIMIT)
-    {
-      /* The dbus-daemon will have inherited our large rlimit */
-      g_test_skip ("Cannot test, our own fd limit was already large");
-      return;
-    }
-
-  if (prlimit (f->daemon_pid, RLIMIT_NOFILE, NULL, &lim) < 0)
-    g_error ("prlimit(): %s", g_strerror (errno));
-
-  g_test_message ("dbus-daemon's RLIMIT_NOFILE: rlim_cur: %ld, rlim_max: %ld",
-                  (long) lim.rlim_cur, (long) lim.rlim_max);
-
-  if (lim.rlim_cur != RLIM_INFINITY)
-    g_assert_cmpint (lim.rlim_cur, >=, DESIRED_RLIMIT);
-
-#else /* !HAVE_PRLIMIT */
-
-  g_test_skip ("prlimit() not supported on this platform");
-
-#endif /* !HAVE_PRLIMIT */
+  dbus_connection_close (third_conn);
+  dbus_connection_unref (third_conn);
 }
+
+static void
+test_max_replies_per_connection (Fixture *f,
+    gconstpointer context)
+{
+  GQueue received = G_QUEUE_INIT;
+  GQueue errors = G_QUEUE_INIT;
+  DBusMessage *m;
+  DBusPendingCall *pc;
+  guint i;
+  DBusError error = DBUS_ERROR_INIT;
+
+  if (f->skip)
+    return;
+
+  add_hold_filter (f);
+
+  /* The configured limit is 3 replies per connection. */
+  for (i = 0; i < 3; i++)
+    {
+      m = dbus_message_new_method_call (
+          dbus_bus_get_unique_name (f->right_conn), "/",
+          "com.example", "Spam");
+
+      if (m == NULL)
+        g_error ("OOM");
+
+      if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
+                                            DBUS_TIMEOUT_INFINITE) ||
+          pc == NULL)
+        g_error ("OOM");
+
+      if (dbus_pending_call_get_completed (pc))
+        pc_enqueue (pc, &received);
+      else if (!dbus_pending_call_set_notify (pc, pc_enqueue, &received,
+            NULL))
+        g_error ("OOM");
+
+      dbus_pending_call_unref (pc);
+      dbus_message_unref (m);
+    }
+
+  while (g_queue_get_length (&f->held_messages) < 3)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  g_assert_cmpuint (g_queue_get_length (&received), ==, 0);
+  g_assert_cmpuint (g_queue_get_length (&errors), ==, 0);
+
+  /* Go a couple of messages over the limit. */
+  for (i = 0; i < 2; i++)
+    {
+      m = dbus_message_new_method_call (
+          dbus_bus_get_unique_name (f->right_conn), "/",
+          "com.example", "Spam");
+
+      if (m == NULL)
+        g_error ("OOM");
+
+      if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
+                                            DBUS_TIMEOUT_INFINITE) ||
+          pc == NULL)
+        g_error ("OOM");
+
+      if (dbus_pending_call_get_completed (pc))
+        pc_enqueue (pc, &errors);
+      else if (!dbus_pending_call_set_notify (pc, pc_enqueue, &errors,
+            NULL))
+        g_error ("OOM");
+
+      dbus_pending_call_unref (pc);
+      dbus_message_unref (m);
+    }
+
+  /* Reply to the held messages. */
+  for (m = g_queue_pop_head (&f->held_messages);
+       m != NULL;
+       m = g_queue_pop_head (&f->held_messages))
+    {
+      DBusMessage *reply = dbus_message_new_method_return (m);
+
+      if (reply == NULL)
+        g_error ("OOM");
+
+      if (!dbus_connection_send (f->right_conn, reply, NULL))
+        g_error ("OOM");
+
+      dbus_message_unref (reply);
+    }
+
+  /* Wait for all 5 replies to come in. */
+  while (g_queue_get_length (&received) + g_queue_get_length (&errors) < 5)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  /* The first three succeeded. */
+  for (i = 0; i < 3; i++)
+    {
+      m = g_queue_pop_head (&received);
+      g_assert (m != NULL);
+
+      if (dbus_set_error_from_message (&error, m))
+        g_error ("Unexpected error: %s: %s", error.name, error.message);
+
+      dbus_message_unref (m);
+    }
+
+  /* The last two failed. */
+  for (i = 0; i < 2; i++)
+    {
+      m = g_queue_pop_head (&errors);
+      g_assert (m != NULL);
+
+      if (!dbus_set_error_from_message (&error, m))
+        g_error ("Unexpected success");
+
+      g_assert_cmpstr (error.name, ==, DBUS_ERROR_LIMITS_EXCEEDED);
+      dbus_error_free (&error);
+      dbus_message_unref (m);
+    }
+
+  g_assert_cmpuint (g_queue_get_length (&received), ==, 0);
+  g_assert_cmpuint (g_queue_get_length (&errors), ==, 0);
+  g_queue_clear (&received);
+  g_queue_clear (&errors);
+}
+
+static void
+test_max_match_rules_per_connection (Fixture *f,
+    gconstpointer context)
+{
+  DBusError error = DBUS_ERROR_INIT;
+
+  if (f->skip)
+    return;
+
+  dbus_bus_add_match (f->left_conn, "sender='com.example.C1'", &error);
+  test_assert_no_error (&error);
+  dbus_bus_add_match (f->left_conn, "sender='com.example.C2'", &error);
+  test_assert_no_error (&error);
+  dbus_bus_add_match (f->left_conn, "sender='com.example.C3'", &error);
+  test_assert_no_error (&error);
+
+  dbus_bus_add_match (f->left_conn, "sender='com.example.C4'", &error);
+  g_assert_cmpstr (error.name, ==, DBUS_ERROR_LIMITS_EXCEEDED);
+  dbus_error_free (&error);
+
+  dbus_bus_remove_match (f->left_conn, "sender='com.example.C3'", &error);
+  test_assert_no_error (&error);
+
+  dbus_bus_add_match (f->left_conn, "sender='com.example.C4'", &error);
+  test_assert_no_error (&error);
+}
+
+static void
+test_max_names_per_connection (Fixture *f,
+    gconstpointer context)
+{
+  DBusError error = DBUS_ERROR_INIT;
+  int ret;
+
+  if (f->skip)
+    return;
+
+  /* The limit in the configuration file is set to 4, but we only own 3
+   * names here - remember that the unique name is a name too. */
+
+  ret = dbus_bus_request_name (f->left_conn, "com.example.C1", 0, &error);
+  test_assert_no_error (&error);
+  g_assert_cmpint (ret, ==, DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER);
+
+  ret = dbus_bus_request_name (f->left_conn, "com.example.C2", 0, &error);
+  test_assert_no_error (&error);
+  g_assert_cmpint (ret, ==, DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER);
+
+  ret = dbus_bus_request_name (f->left_conn, "com.example.C3", 0, &error);
+  test_assert_no_error (&error);
+  g_assert_cmpint (ret, ==, DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER);
+
+  ret = dbus_bus_request_name (f->left_conn, "com.example.C4", 0, &error);
+  g_assert_cmpstr (error.name, ==, DBUS_ERROR_LIMITS_EXCEEDED);
+  dbus_error_free (&error);
+  g_assert_cmpint (ret, ==, -1);
+
+  ret = dbus_bus_release_name (f->left_conn, "com.example.C3", &error);
+  test_assert_no_error (&error);
+  g_assert_cmpint (ret, ==, DBUS_RELEASE_NAME_REPLY_RELEASED);
+
+  ret = dbus_bus_request_name (f->left_conn, "com.example.C4", 0, &error);
+  test_assert_no_error (&error);
+  g_assert_cmpint (ret, ==, DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER);
+}
+
+#if defined(DBUS_UNIX) && defined(HAVE_UNIX_FD_PASSING) && defined(HAVE_GIO_UNIX)
+
+static DBusHandlerResult
+wait_for_disconnected_cb (DBusConnection *client_conn,
+    DBusMessage *message,
+    void *data)
+{
+  gboolean *disconnected = data;
+
+  if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected"))
+    {
+      *disconnected = TRUE;
+      return DBUS_HANDLER_RESULT_HANDLED;
+    }
+
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+static const guchar partial_message[] =
+{
+  DBUS_LITTLE_ENDIAN,
+  DBUS_MESSAGE_TYPE_METHOD_CALL,
+  0, /* flags */
+  1, /* version */
+  0xff, 0xff, 0, 0, /* length of body = 65535 bytes */
+  1, 2, 3, 4, /* cookie */
+  0xff, 0xff, 0, 0, /* length of header fields array = 65535 bytes */
+  42 /* pretending to be the beginning of the header fields array */
+};
+
+static void
+send_all_with_fd (GSocket *socket,
+                  const guchar *local_partial_message,
+                  gsize len,
+                  int fd)
+{
+  GSocketControlMessage *fdm = g_unix_fd_message_new ();
+  GError *error = NULL;
+  gssize sent;
+  GOutputVector vector = { local_partial_message, len };
+
+  g_unix_fd_message_append_fd (G_UNIX_FD_MESSAGE (fdm), fd, &error);
+  g_assert_no_error (error);
+
+  sent = g_socket_send_message (socket, NULL, &vector, 1, &fdm, 1,
+      G_SOCKET_MSG_NONE, NULL, &error);
+  g_assert_no_error (error);
+  g_assert_cmpint (sent, >=, 1);
+  g_assert_cmpint (sent, <=, vector.size);
+
+  while (((gsize) sent) < vector.size)
+    {
+      vector.size -= sent;
+      vector.buffer = ((const guchar *) vector.buffer) + sent;
+      sent = g_socket_send_message (socket, NULL, &vector, 1, NULL, 0,
+          G_SOCKET_MSG_NONE, NULL, &error);
+      g_assert_no_error (error);
+      g_assert_cmpint (sent, >=, 1);
+      g_assert_cmpint (sent, <=, vector.size);
+    }
+
+  g_object_unref (fdm);
+}
+
+static void
+test_pending_fd_timeout (Fixture *f,
+    gconstpointer context)
+{
+  GError *error = NULL;
+  gint64 start;
+  int fd;
+  GSocket *socket;
+  gboolean have_mem;
+  gboolean disconnected = FALSE;
+
+  if (f->skip)
+    return;
+
+  if (getuid () == 0)
+    {
+      g_test_skip ("Cannot test, uid 0 is immune to this limit");
+      return;
+    }
+
+  have_mem = dbus_connection_add_filter (f->left_conn, wait_for_disconnected_cb,
+      &disconnected, NULL);
+  g_assert (have_mem);
+
+  /* This is not API. Never do this. */
+
+  if (!dbus_connection_get_socket (f->left_conn, &fd))
+    g_error ("failed to steal fd from left connection");
+
+  socket = g_socket_new_from_fd (fd, &error);
+  g_assert_no_error (error);
+  g_assert (socket != NULL);
+
+  /* We send part of a message that contains a fd, then stop. */
+  start = g_get_monotonic_time ();
+  send_all_with_fd (socket, partial_message, G_N_ELEMENTS (partial_message),
+                    fd);
+
+  while (!disconnected)
+    {
+      test_progress ('.');
+      test_main_context_iterate (f->ctx, TRUE);
+
+      /* It should take no longer than 500ms to get disconnected. We'll
+       * be generous and allow 1000ms. */
+      g_assert_cmpint (g_get_monotonic_time (), <=, start + G_USEC_PER_SEC);
+    }
+
+  g_object_unref (socket);
+}
+
 #endif
 
 static void
@@ -743,6 +1058,15 @@ teardown (Fixture *f,
           f->right_conn_echo = FALSE;
         }
 
+      if (f->right_conn_hold)
+        {
+          dbus_connection_remove_filter (f->right_conn, hold_filter, f);
+          f->right_conn_hold = FALSE;
+        }
+
+      g_queue_foreach (&f->held_messages, (GFunc) dbus_message_unref, NULL);
+      g_queue_clear (&f->held_messages);
+
       dbus_connection_close (f->right_conn);
       dbus_connection_unref (f->right_conn);
       f->right_conn = NULL;
@@ -776,29 +1100,55 @@ teardown (Fixture *f,
     }
 
   test_main_context_unref (f->ctx);
+  g_free (f->address);
 }
 
 static Config limited_config = {
     "34393", 10000, "valid-config-files/incoming-limit.conf",
-    TEST_USER_ME, SPECIFY_ADDRESS
+    SPECIFY_ADDRESS
 };
 
 static Config finite_timeout_config = {
     NULL, 1, "valid-config-files/finite-timeout.conf",
-    TEST_USER_ME, SPECIFY_ADDRESS
+    SPECIFY_ADDRESS
 };
 
 #ifdef DBUS_UNIX
 static Config listen_unix_runtime_config = {
     "61303", 1, "valid-config-files/listen-unix-runtime.conf",
-    TEST_USER_ME, RELY_ON_DEFAULT
+    RELY_ON_DEFAULT
+};
+#endif
+
+static Config max_completed_connections_config = {
+    NULL, 1, "valid-config-files/max-completed-connections.conf",
+    SPECIFY_ADDRESS
 };
 
-static Config as_another_user_config = {
-    NULL, 1, "valid-config-files/as-another-user.conf",
-    /* We start the dbus-daemon as root and drop privileges, like the
-     * real system bus does */
-    TEST_USER_ROOT, SPECIFY_ADDRESS
+static Config max_connections_per_user_config = {
+    NULL, 1, "valid-config-files/max-connections-per-user.conf",
+    SPECIFY_ADDRESS
+};
+
+static Config max_replies_per_connection_config = {
+    NULL, 1, "valid-config-files/max-replies-per-connection.conf",
+    SPECIFY_ADDRESS
+};
+
+static Config max_match_rules_per_connection_config = {
+    NULL, 1, "valid-config-files/max-match-rules-per-connection.conf",
+    SPECIFY_ADDRESS
+};
+
+static Config max_names_per_connection_config = {
+    NULL, 1, "valid-config-files/max-names-per-connection.conf",
+    SPECIFY_ADDRESS
+};
+
+#if defined(DBUS_UNIX) && defined(HAVE_UNIX_FD_PASSING) && defined(HAVE_GIO_UNIX)
+static Config pending_fd_timeout_config = {
+    NULL, 1, "valid-config-files/pending-fd-timeout.conf",
+    SPECIFY_ADDRESS
 };
 #endif
 
@@ -819,17 +1169,34 @@ main (int argc,
   g_test_add ("/processid", Fixture, NULL, setup, test_processid, teardown);
   g_test_add ("/canonical-path/uae", Fixture, NULL,
       setup, test_canonical_path_uae, teardown);
+  g_test_add ("/limits/max-completed-connections", Fixture,
+      &max_completed_connections_config,
+      setup, test_max_connections, teardown);
+  g_test_add ("/limits/max-connections-per-user", Fixture,
+      &max_connections_per_user_config,
+      setup, test_max_connections, teardown);
+  g_test_add ("/limits/max-replies-per-connection", Fixture,
+      &max_replies_per_connection_config,
+      setup, test_max_replies_per_connection, teardown);
+  g_test_add ("/limits/max-match-rules-per-connection", Fixture,
+      &max_match_rules_per_connection_config,
+      setup, test_max_match_rules_per_connection, teardown);
+  g_test_add ("/limits/max-names-per-connection", Fixture,
+      &max_names_per_connection_config,
+      setup, test_max_names_per_connection, teardown);
+
+#if defined(DBUS_UNIX) && defined(HAVE_UNIX_FD_PASSING) && defined(HAVE_GIO_UNIX)
+  g_test_add ("/limits/pending-fd-timeout", Fixture,
+      &pending_fd_timeout_config,
+      setup, test_pending_fd_timeout, teardown);
+#endif
+
 #ifdef DBUS_UNIX
   /* We can't test this in loopback.c with the rest of unix:runtime=yes,
    * because dbus_bus_get[_private] is the only way to use the default,
    * and that blocks on a round-trip to the dbus-daemon */
   g_test_add ("/unix-runtime-is-default", Fixture, &listen_unix_runtime_config,
       setup, test_echo, teardown);
-
-  g_test_add ("/fd-limit/session", Fixture, NULL,
-              setup, test_fd_limit, teardown);
-  g_test_add ("/fd-limit/system", Fixture, &as_another_user_config,
-              setup, test_fd_limit, teardown);
 #endif
 
   return g_test_run ();
