@@ -45,6 +45,10 @@
 
 #include <dbus/dbus.h>
 
+#ifdef G_OS_WIN
+# define isatty(x) _isatty(x)
+#endif
+
 void
 _test_assert_no_error (const DBusError *e,
     const char *file,
@@ -89,22 +93,14 @@ child_setup (gpointer user_data)
 static gchar *
 spawn_dbus_daemon (const gchar *binary,
     const gchar *configuration,
+    const gchar *listen_address,
     TestUser user,
     GPid *daemon_pid)
 {
   GError *error = NULL;
   GString *address;
   gint address_fd;
-  const gchar *const argv[] = {
-      binary,
-      configuration,
-      "--nofork",
-      "--print-address=1", /* stdout */
-#ifdef DBUS_UNIX
-      "--systemd-activation",
-#endif
-      NULL
-  };
+  GPtrArray *argv;
 #ifdef DBUS_UNIX
   const struct passwd *pwd = NULL;
 #endif
@@ -114,7 +110,7 @@ spawn_dbus_daemon (const gchar *binary,
 #ifdef DBUS_UNIX
       if (getuid () != 0)
         {
-          g_message ("SKIP: cannot use alternative uid when not uid 0");
+          g_test_skip ("cannot use alternative uid when not uid 0");
           return NULL;
         }
 
@@ -128,7 +124,11 @@ spawn_dbus_daemon (const gchar *binary,
 
             if (pwd == NULL)
               {
-                g_message ("SKIP: user '%s' does not exist", DBUS_USER);
+                gchar *message = g_strdup_printf ("user '%s' does not exist",
+                    DBUS_USER);
+
+                g_test_skip (message);
+                g_free (message);
                 return NULL;
               }
 
@@ -139,7 +139,11 @@ spawn_dbus_daemon (const gchar *binary,
 
             if (pwd == NULL)
               {
-                g_message ("SKIP: user '%s' does not exist", DBUS_TEST_USER);
+                gchar *message = g_strdup_printf ("user '%s' does not exist",
+                    DBUS_TEST_USER);
+
+                g_test_skip (message);
+                g_free (message);
                 return NULL;
               }
 
@@ -149,13 +153,28 @@ spawn_dbus_daemon (const gchar *binary,
             g_assert_not_reached ();
         }
 #else
-      g_message ("SKIP: cannot use alternative uid on Windows");
+      g_test_skip ("cannot use alternative uid on Windows");
       return NULL;
 #endif
     }
 
+  argv = g_ptr_array_new_with_free_func (g_free);
+  g_ptr_array_add (argv, g_strdup (binary));
+  g_ptr_array_add (argv, g_strdup (configuration));
+  g_ptr_array_add (argv, g_strdup ("--nofork"));
+  g_ptr_array_add (argv, g_strdup ("--print-address=1")); /* stdout */
+
+  if (listen_address != NULL)
+    g_ptr_array_add (argv, g_strdup (listen_address));
+
+#ifdef DBUS_UNIX
+  g_ptr_array_add (argv, g_strdup ("--systemd-activation"));
+#endif
+
+  g_ptr_array_add (argv, NULL);
+
   g_spawn_async_with_pipes (NULL, /* working directory */
-      (gchar **) argv, /* g_s_a_w_p() is not const-correct :-( */
+      (gchar **) argv->pdata,
       NULL, /* envp */
       G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
 #ifdef DBUS_UNIX
@@ -170,6 +189,8 @@ spawn_dbus_daemon (const gchar *binary,
       &error);
   g_assert_no_error (error);
 
+  g_ptr_array_free (argv, TRUE);
+
   address = g_string_new (NULL);
 
   /* polling until the dbus-daemon writes out its address is a bit stupid,
@@ -211,26 +232,37 @@ test_get_dbus_daemon (const gchar *config_file,
 {
   gchar *dbus_daemon;
   gchar *arg;
+  const gchar *listen_address = NULL;
   gchar *address;
 
+  /* we often have to override this because on Windows, the default may be
+   * autolaunch:, which is globally-scoped and hence unsuitable for
+   * regression tests */
+  listen_address = "--address=" TEST_LISTEN;
+
   if (config_file != NULL)
     {
 
       if (g_getenv ("DBUS_TEST_DATA") == NULL)
         {
-          g_message ("SKIP: set DBUS_TEST_DATA to a directory containing %s",
+          g_test_message ("set DBUS_TEST_DATA to a directory containing %s",
               config_file);
+          g_test_skip ("DBUS_TEST_DATA not set");
           return NULL;
         }
 
       arg = g_strdup_printf (
           "--config-file=%s/%s",
           g_getenv ("DBUS_TEST_DATA"), config_file);
+
+      /* The configuration file is expected to give a suitable address,
+       * do not override it */
+      listen_address = NULL;
     }
-  else if (g_getenv ("DBUS_TEST_SYSCONFDIR") != NULL)
+  else if (g_getenv ("DBUS_TEST_DATADIR") != NULL)
     {
       arg = g_strdup_printf ("--config-file=%s/dbus-1/session.conf",
-          g_getenv ("DBUS_TEST_SYSCONFDIR"));
+          g_getenv ("DBUS_TEST_DATADIR"));
     }
   else if (g_getenv ("DBUS_TEST_DATA") != NULL)
     {
@@ -252,7 +284,7 @@ test_get_dbus_daemon (const gchar *config_file,
     {
       if (config_file != NULL || user != TEST_USER_ME)
         {
-          g_message ("SKIP: cannot use DBUS_TEST_DAEMON_ADDRESS for "
+          g_test_skip ("cannot use DBUS_TEST_DAEMON_ADDRESS for "
               "unusally-configured dbus-daemon");
           address = NULL;
         }
@@ -263,7 +295,8 @@ test_get_dbus_daemon (const gchar *config_file,
     }
   else
     {
-      address = spawn_dbus_daemon (dbus_daemon, arg, user, daemon_pid);
+      address = spawn_dbus_daemon (dbus_daemon, arg,
+          listen_address, user, daemon_pid);
     }
 
   g_free (dbus_daemon);
@@ -331,8 +364,9 @@ test_connect_to_bus_as_user (TestMainContext *ctx,
 
   if (ruid != 0 || euid != 0 || suid != 0)
     {
-      g_message ("SKIP: not uid 0 (ruid=%ld euid=%ld suid=%ld)",
+      g_test_message ("not uid 0 (ruid=%ld euid=%ld suid=%ld)",
           (unsigned long) ruid, (unsigned long) euid, (unsigned long) suid);
+      g_test_skip ("not uid 0");
       return NULL;
     }
 
@@ -340,7 +374,8 @@ test_connect_to_bus_as_user (TestMainContext *ctx,
 
   if (pwd == NULL)
     {
-      g_message ("SKIP: getpwnam(\"%s\"): %s", username, g_strerror (errno));
+      g_test_message ("getpwnam(\"%s\"): %s", username, g_strerror (errno));
+      g_test_skip ("not uid 0");
       return NULL;
     }
 
@@ -366,7 +401,7 @@ test_connect_to_bus_as_user (TestMainContext *ctx,
         return test_connect_to_bus (ctx, address);
 
       default:
-        g_message ("SKIP: setresuid() not available, or unsure about "
+        g_test_skip ("setresuid() not available, or unsure about "
             "credentials-passing semantics on this platform");
         return NULL;
     }
@@ -394,6 +429,8 @@ time_out (gpointer data)
 }
 
 #ifdef G_OS_UNIX
+static void wrap_abort (int signal) _DBUS_GNUC_NORETURN;
+
 static void
 wrap_abort (int signal)
 {
@@ -428,3 +465,10 @@ test_init (int *argcp, char ***argvp)
     }
 #endif
 }
+
+void
+test_progress (char symbol)
+{
+  if (g_test_verbose () && isatty (1))
+    g_print ("%c", symbol);
+}
