@@ -164,16 +164,20 @@ verbose (const char *format,
 #endif /* DBUS_ENABLE_VERBOSE_MODE */
 }
 
+static void usage (int ecode) _DBUS_GNUC_NORETURN;
+
 static void
 usage (int ecode)
 {
   fprintf (stderr, "dbus-launch [--version] [--help] [--sh-syntax]"
            " [--csh-syntax] [--auto-syntax] [--binary-syntax] [--close-stderr]"
-           " [--exit-with-session] [--autolaunch=MACHINEID]"
+           " [--exit-with-session|--exit-with-x11] [--autolaunch=MACHINEID]"
            " [--config-file=FILENAME] [PROGRAM] [ARGS...]\n");
   exit (ecode);
 }
 
+static void version (void) _DBUS_GNUC_NORETURN;
+
 static void
 version (void)
 {
@@ -205,6 +209,7 @@ xstrdup (const char *str)
   return copy;
 }
 
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 static char *
 concat2 (const char *a,
     const char *b)
@@ -224,6 +229,7 @@ concat2 (const char *a,
   memcpy (ret + la, b, lb + 1);
   return ret;
 }
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
 
 typedef enum
 {
@@ -468,6 +474,8 @@ signal_handler (int sig)
     }
 }
 
+static void kill_bus_when_session_ends (void) _DBUS_GNUC_NORETURN;
+
 static void
 kill_bus_when_session_ends (void)
 {
@@ -605,15 +613,16 @@ kill_bus_when_session_ends (void)
             }
         }
     }
+  /* not reached */
 }
 
-static void
+_DBUS_GNUC_NORETURN static void
 babysit (int   exit_with_session,
          pid_t child_pid,
          int   read_bus_pid_fd)  /* read pid from here */
 {
+  DBusEnsureStandardFdsFlags flags;
   int ret;
-  int dev_null_fd;
   const char *s;
 
   verbose ("babysitting, exit_with_session = %d, child_pid = %ld, read_bus_pid_fd = %d\n",
@@ -633,28 +642,26 @@ babysit (int   exit_with_session,
       exit (1);
     }
 
+  flags = DBUS_FORCE_STDOUT_NULL;
+
+  if (!exit_with_session)
+    flags |= DBUS_FORCE_STDIN_NULL;
+
+  s = getenv ("DBUS_DEBUG_OUTPUT");
+
+  if (s == NULL || *s == '\0')
+    flags |= DBUS_FORCE_STDERR_NULL;
+
   /* Close stdout/stderr so we don't block an "eval" or otherwise
    * lock up. stdout is still chaining through to dbus-launch
    * and in turn to the parent shell.
    */
-  dev_null_fd = open ("/dev/null", O_RDWR);
-  if (dev_null_fd >= 0)
+  if (!_dbus_ensure_standard_fds (flags, &s))
     {
-      if (!exit_with_session)
-        dup2 (dev_null_fd, 0);
-      dup2 (dev_null_fd, 1);
-      s = getenv ("DBUS_DEBUG_OUTPUT");
-      if (s == NULL || *s == '\0')
-        dup2 (dev_null_fd, 2);
-      close (dev_null_fd);
+      fprintf (stderr, "%s: %s\n", s, strerror (errno));
+      exit (1);
     }
-  else
-    {
-      fprintf (stderr, "Failed to open /dev/null: %s\n",
-               strerror (errno));
-      /* continue, why not */
-    }
-  
+
   ret = fork ();
 
   if (ret < 0)
@@ -725,29 +732,23 @@ babysit (int   exit_with_session,
 static void
 do_close_stderr (void)
 {
-  int fd;
+  const char *err;
 
   fflush (stderr);
 
-  /* dbus-launch is a Unix-only program, so we can rely on /dev/null being there.
-   * We're including unistd.h and we're dealing with sh/csh launch sequences...
-   */
-  fd = open ("/dev/null", O_RDWR);
-  if (fd == -1)
+  if (!_dbus_ensure_standard_fds (DBUS_FORCE_STDERR_NULL, &err))
     {
-      fprintf (stderr, "Internal error: cannot open /dev/null: %s", strerror (errno));
+      fprintf (stderr, "%s: %s\n", err, strerror (errno));
       exit (1);
     }
-
-  close (2);
-  if (dup2 (fd, 2) == -1)
-    {
-      /* error; we can't report an error anymore... */
-      exit (1);
-    }
-  close (fd);
 }
 
+static void pass_info (const char *runprog, const char *bus_address,
+                       pid_t bus_pid, long bus_wid, int c_shell_syntax,
+                       int bourne_shell_syntax, int binary_syntax,
+                       int argc, char **argv,
+                       int remaining_args) _DBUS_GNUC_NORETURN;
+
 static void
 pass_info (const char *runprog, const char *bus_address, pid_t bus_pid,
            long bus_wid, int c_shell_syntax, int bourne_shell_syntax,
@@ -827,6 +828,7 @@ main (int argc, char **argv)
   const char *runprog = NULL;
   int remaining_args = 0;
   int exit_with_session;
+  int exit_with_x11 = FALSE;
   int binary_syntax = FALSE;
   int c_shell_syntax = FALSE;
   int bourne_shell_syntax = FALSE;
@@ -885,6 +887,8 @@ main (int argc, char **argv)
         version ();
       else if (strcmp (arg, "--exit-with-session") == 0)
         exit_with_session = TRUE;
+      else if (strcmp (arg, "--exit-with-x11") == 0)
+        exit_with_x11 = TRUE;
       else if (strcmp (arg, "--close-stderr") == 0)
         close_stderr = TRUE;
       else if (strstr (arg, "--autolaunch=") == arg)
@@ -996,6 +1000,9 @@ main (int argc, char **argv)
   if (exit_with_session)
     verbose ("--exit-with-session enabled\n");
 
+  if (exit_with_x11)
+    verbose ("--exit-with-x11 enabled\n");
+
   if (autolaunch)
     {      
 #ifndef DBUS_BUILD_X11
@@ -1050,10 +1057,10 @@ main (int argc, char **argv)
         }
 
       verbose ("Autolaunch enabled (using X11).\n");
-      if (!exit_with_session)
+      if (!exit_with_x11)
 	{
-	  verbose ("--exit-with-session automatically enabled\n");
-	  exit_with_session = TRUE;
+          verbose ("--exit-with-x11 automatically enabled\n");
+          exit_with_x11 = TRUE;
 	}
 
       if (!x11_init ())
@@ -1076,12 +1083,33 @@ main (int argc, char **argv)
 	  exit (0);
 	}
 #endif /* DBUS_ENABLE_X11_AUTOLAUNCH */
+#endif /* DBUS_BUILD_X11 */
     }
+  else if (exit_with_x11)
+    {
+#ifndef DBUS_BUILD_X11
+      fprintf (stderr, "Session lifetime based on X11 requested, but X11 support not compiled in.\n");
+      exit (1);
+#else /* DBUS_BUILD_X11 */
+      if (!read_machine_uuid_if_needed())
+        {
+          fprintf (stderr, "Session lifetime based on X11 requested, but machine UUID unavailable.\n");
+          exit (1);
+        }
+
+      if (!x11_init ())
+        {
+          fprintf (stderr, "Session lifetime based on X11 requested, but X11 initialization failed.\n");
+          exit (1);
+        }
+#endif /* DBUS_BUILD_X11 */
+    }
+#ifdef DBUS_BUILD_X11
   else if (read_machine_uuid_if_needed())
     {
       x11_init();
-#endif /* DBUS_BUILD_X11 */
     }
+#endif /* DBUS_BUILD_X11 */
 
 
   if (pipe (bus_pid_to_launcher_pipe) < 0 ||
@@ -1143,7 +1171,7 @@ main (int argc, char **argv)
            * and will also reap the pre-forked bus
            * daemon
            */
-          babysit (exit_with_session, ret,
+          babysit (exit_with_session || exit_with_x11, ret,
                    bus_pid_to_babysitter_pipe[READ_END]);
           exit (0);
         }
@@ -1184,10 +1212,10 @@ main (int argc, char **argv)
  
 #ifdef DBUS_ENABLE_EMBEDDED_TESTS
       {
-        const char *test_daemon;
         /* exec from testdir */
-        if (getenv ("DBUS_USE_TEST_BINARY") != NULL &&
-            (test_daemon = getenv ("DBUS_TEST_DAEMON")) != NULL)
+        const char *test_daemon = getenv ("DBUS_TEST_DAEMON");
+
+        if (test_daemon != NULL)
           {
             if (config_file == NULL && getenv ("DBUS_TEST_DATA") != NULL)
               {
@@ -1203,6 +1231,7 @@ main (int argc, char **argv)
 
             execl (test_daemon,
                    test_daemon,
+                   close_stderr ? "--syslog-only" : "--syslog",
                    "--fork",
                    "--print-pid", write_pid_fd_as_string,
                    "--print-address", write_address_fd_as_string,
@@ -1220,6 +1249,7 @@ main (int argc, char **argv)
 
       execl (DBUS_DAEMONDIR"/dbus-daemon",
              DBUS_DAEMONDIR"/dbus-daemon",
+             close_stderr ? "--syslog-only" : "--syslog",
              "--fork",
              "--print-pid", write_pid_fd_as_string,
              "--print-address", write_address_fd_as_string,
@@ -1239,6 +1269,7 @@ main (int argc, char **argv)
        */
       execlp ("dbus-daemon",
               "dbus-daemon",
+              close_stderr ? "--syslog-only" : "--syslog",
               "--fork",
               "--print-pid", write_pid_fd_as_string,
               "--print-address", write_address_fd_as_string,
