@@ -65,6 +65,8 @@
 
 static const char me[] = "dbus-run-session";
 
+static void usage (int ecode) _DBUS_GNUC_NORETURN;
+
 static void
 usage (int ecode)
 {
@@ -81,6 +83,8 @@ usage (int ecode)
   exit (ecode);
 }
 
+static void version (void) _DBUS_GNUC_NORETURN;
+
 static void
 version (void)
 {
@@ -95,6 +99,8 @@ version (void)
   exit (0);
 }
 
+static void oom (void) _DBUS_GNUC_NORETURN;
+
 static void
 oom (void)
 {
@@ -191,6 +197,8 @@ exec_dbus_daemon (const char *dbus_daemon,
            me, dbus_daemon, strerror (errno));
 }
 
+static void exec_app (int prog_arg, char **argv) _DBUS_GNUC_NORETURN;
+
 static void
 exec_app (int prog_arg, char **argv)
 {
